import os
import epitran as epitran_module

import cloudlanguagetools.service
import cloudlanguagetools.constants
import cloudlanguagetools.languages
import cloudlanguagetools.transliterationlanguage


class EpitranTransliterationLanguage(cloudlanguagetools.transliterationlanguage.TransliterationLanguage):
    def __init__(self, language, epitran_language_code):
        self.service = cloudlanguagetools.constants.Service.Epitran
        self.language = language
        self.epitran_language_code = epitran_language_code

    def get_transliteration_name(self):
        result = f'{self.language.lang_name} (IPA Pronunciation), {self.service.name} ({self.epitran_language_code})'
        return result

    def get_transliteration_shortname(self):
        result = f'IPA Pronunciation, {self.service.name} ({self.epitran_language_code})'
        return result        

    def get_transliteration_key(self):
        key = {
            'language_code': self.epitran_language_code
        }
        return key

class EpitranService(cloudlanguagetools.service.Service):
    def __init__(self):
        pass

    def get_tts_voice_list(self):
        return []

    def get_translation_language_list(self):
        return []

    def get_transliteration_language_list(self):
        result = [
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.am, 'amh-Ethi'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.ar, 'ara-Arab'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.az, 'aze-Cyrl'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.az, 'aze-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.ca, 'cat-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.ceb, 'ceb-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.de, 'deu-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.de, 'deu-Latn-np'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.de, 'deu-Latn-nar'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.en, 'eng-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.fr, 'fra-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.fr, 'fra-Latn-np'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.ha, 'hau-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.hi, 'hin-Deva'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.hu, 'hun-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.id_, 'ind-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.it, 'ita-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.jw, 'jav-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.kk, 'kaz-Cyrl'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.kk, 'kaz-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.rw, 'kin-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.ky, 'kir-Arab'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.ky, 'kir-Cyrl'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.ky, 'kir-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.lo, 'lao-Laoo'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.mr, 'mar-Deva'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.mt, 'mlt-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.ms, 'msa-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.nl, 'nld-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.ny, 'nya-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.pa, 'pan-Guru'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.pl, 'pol-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.ro, 'ron-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.ru, 'rus-Cyrl'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.sn, 'sna-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.so, 'som-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.es, 'spa-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.sw, 'swa-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.sv, 'swe-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.ta, 'tam-Taml'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.te, 'tel-Telu'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.tg, 'tgk-Cyrl'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.th, 'tha-Thai'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.ti, 'tir-Ethi'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.tk, 'tuk-Cyrl'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.tk, 'tuk-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.tr, 'tur-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.uk, 'ukr-Cyrl'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.ug, 'uig-Arab'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.uz, 'uzb-Cyrl'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.uz, 'uzb-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.vi, 'vie-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.xh, 'xho-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.yo, 'yor-Latn'),
            EpitranTransliterationLanguage(cloudlanguagetools.languages.Language.zu, 'zul-Latn'),            

        ]
        return result

    def get_transliteration(self, text, transliteration_key):
        epi = epitran_module.Epitran(transliteration_key['language_code'])
        result = epi.transliterate(text)
        return result



import json
import requests
import tempfile
import logging
import time

import cloudlanguagetools.service
import cloudlanguagetools.constants
import cloudlanguagetools.languages
import cloudlanguagetools.ttsvoice
import cloudlanguagetools.translationlanguage
import cloudlanguagetools.transliterationlanguage
import cloudlanguagetools.errors

class VoicenVoice(cloudlanguagetools.ttsvoice.TtsVoice):
    def __init__(self, voice_id, audio_language, gender, name):
        self.service = cloudlanguagetools.constants.Service.Voicen
        self.audio_language = audio_language
        self.name = name
        self.gender = gender
        self.voice_id = voice_id


    def get_voice_key(self):
        return {
            'voice_id': self.voice_id,
            'lang': self.audio_language.lang.name
        }

    def get_voice_shortname(self):
        return f'{self.name}'

    def get_options(self):
        return {}

class VoicenService(cloudlanguagetools.service.Service):
    def __init__(self):
        pass

    def configure(self, config):
        self.api_key = config['key']

    def get_translation_language_list(self):
        return []

    def get_tts_voice_list(self):
        result = [
            VoicenVoice('325640', cloudlanguagetools.languages.AudioLanguage.az_AZ, cloudlanguagetools.constants.Gender.Female, 'Aytac'),
            VoicenVoice('325641', cloudlanguagetools.languages.AudioLanguage.az_AZ, cloudlanguagetools.constants.Gender.Female, 'Aynur'),
            VoicenVoice('325642', cloudlanguagetools.languages.AudioLanguage.az_AZ, cloudlanguagetools.constants.Gender.Male, 'Ramin'),
            VoicenVoice('325643', cloudlanguagetools.languages.AudioLanguage.az_AZ, cloudlanguagetools.constants.Gender.Male, 'Elchin'),
            VoicenVoice('325648', cloudlanguagetools.languages.AudioLanguage.az_AZ, cloudlanguagetools.constants.Gender.Male, 'Kamil'),
            VoicenVoice('325647', cloudlanguagetools.languages.AudioLanguage.tr_TR, cloudlanguagetools.constants.Gender.Female, 'Zeynep'),
            VoicenVoice('325646', cloudlanguagetools.languages.AudioLanguage.tr_TR, cloudlanguagetools.constants.Gender.Male, 'Mesut'),
            VoicenVoice('325644', cloudlanguagetools.languages.AudioLanguage.tr_TR, cloudlanguagetools.constants.Gender.Female, 'Sibel'),
            VoicenVoice('325645', cloudlanguagetools.languages.AudioLanguage.ru_RU, cloudlanguagetools.constants.Gender.Female, 'Anna'),
        ]
        return result

    def get_headers(self):
        headers = {
            'Content-Type': 'application/json',
            'Authorization': f'Token {self.api_key}'
        }
        return headers

    def job_status_ready(self, job_id):
        check_status_url = f'https://tts.voicen.com/api/v1/jobs/{job_id}/'
        response = requests.get(check_status_url, headers=self.get_headers(), timeout=cloudlanguagetools.constants.RequestTimeout)
        status = response.json()['data']['status']        
        if status == 'ready':
            return True
        if status == 'failed':
            error_message = f"Job {job_id} status failed"
            raise cloudlanguagetools.errors.RequestError(error_message)
        return False


    def get_tts_audio(self, text, voice_key, options):
        output_temp_file = tempfile.NamedTemporaryFile()
        output_temp_filename = output_temp_file.name

        # create the audio request
        # ========================
        request_url = 'https://tts.voicen.com/api/v1/jobs/text/'
        data = {
            'text': text,
            'lang': voice_key['lang'],
            'voice_id': voice_key['voice_id']
        }

        logging.info(f'requesting audio for {text}, voice_key {voice_key}')
        response = requests.post(request_url, json=data, headers=self.get_headers(), timeout=cloudlanguagetools.constants.RequestTimeout)
        if response.status_code != 200:
            error_message = f"Status code: {response.status_code} reason: {response.reason}"
            raise cloudlanguagetools.errors.RequestError(error_message)


        response_data = response.json()
        job_id = response_data['data']['id']

        # wait for job to be ready
        # ========================
        total_tries = 7
        max_tries = total_tries
        wait_time = 0.2        
        job_ready = False
        while job_ready == False and max_tries > 0:
            time.sleep(wait_time)            
            logging.debug(f'checking whether job_id {job_id} is ready')
            job_ready = self.job_status_ready(job_id)
            wait_time = wait_time * 2
            max_tries -= 1             

        # retrieve audio
        # ==============

        retrieve_url = f'https://tts.voicen.com/api/v1/jobs/{job_id}/synthesize/'
        logging.info(f'retrieving result from url {retrieve_url}')
        response = requests.get(retrieve_url, headers=self.get_headers())

        if response.status_code == 200:
            with open(output_temp_filename, 'wb') as audio:
                audio.write(response.content)
            return output_temp_file            

        # otherwise, an error occured
        error_message = f"Status code: {response.status_code} reason: {response.reason} voice: [{voice_name}]]"
        raise cloudlanguagetools.errors.RequestError(error_message)


    def get_transliteration_language_list(self):
        return []




class ApiKeyNotFoundError(ValueError):
    pass

class RequestError(ValueError):
    pass

class TimeoutError(ValueError):
    pass

class NotFoundError(ValueError):
    pass

class OverQuotaError(Exception):
    pass



class Service():
    def __init__(self):
        pass

    # used for pre-loading models
    def load_data(self):
        pass

    def get_tts_voice_list(self):
        return []

    def get_translation_language_list(self):
        return []

    def get_transliteration_language_list(self):
        return []

    def get_tokenization_options(self):
        return []


import json
import requests
import tempfile
import logging
import os

import cloudlanguagetools.service
import cloudlanguagetools.constants
import cloudlanguagetools.languages
import cloudlanguagetools.ttsvoice
import cloudlanguagetools.translationlanguage
import cloudlanguagetools.transliterationlanguage
import cloudlanguagetools.errors


class DeepLTranslationLanguage(cloudlanguagetools.translationlanguage.TranslationLanguage):
    def __init__(self, language, language_id):
        self.service = cloudlanguagetools.constants.Service.DeepL
        self.language = language
        self.language_id = language_id

    def get_language_id(self):
        return self.language_id


class DeepLService(cloudlanguagetools.service.Service):
    def __init__(self):
        self.base_url = 'https://api.deepl.com/v2/translate'

    def configure(self, config):
        self.api_key = config['key']
    
    def get_tts_voice_list(self):
        return []


    def get_translation_language_list(self):
        language = cloudlanguagetools.languages.Language
        result = [
            DeepLTranslationLanguage(language.bg, 'BG'),
            DeepLTranslationLanguage(language.cs, 'CS'),
            DeepLTranslationLanguage(language.da, 'DA'),
            DeepLTranslationLanguage(language.de, 'DE'),
            DeepLTranslationLanguage(language.el, 'EL'),
            DeepLTranslationLanguage(language.en, 'EN'),
            DeepLTranslationLanguage(language.es, 'ES'),
            DeepLTranslationLanguage(language.et, 'ET'),
            DeepLTranslationLanguage(language.fi, 'FI'),
            DeepLTranslationLanguage(language.fr, 'FR'),
            DeepLTranslationLanguage(language.hu, 'HU'),
            DeepLTranslationLanguage(language.it, 'IT'),
            DeepLTranslationLanguage(language.ja, 'JA'),
            DeepLTranslationLanguage(language.lt, 'LT'),
            DeepLTranslationLanguage(language.lv, 'LV'),
            DeepLTranslationLanguage(language.nl, 'NL'),
            DeepLTranslationLanguage(language.pl, 'PL'),
            DeepLTranslationLanguage(language.pt_pt, 'PT'),
            DeepLTranslationLanguage(language.ro, 'RO'),
            DeepLTranslationLanguage(language.ru, 'RU'),
            DeepLTranslationLanguage(language.sk, 'SK'),
            DeepLTranslationLanguage(language.sl, 'SL'),
            DeepLTranslationLanguage(language.sv, 'SV'),
            DeepLTranslationLanguage(language.zh_cn, 'ZH'),
        ]
        return result        


    def get_tts_voice_list(self):
        result = []
        return result


    def get_transliteration_language_list(self):
        return []

    def get_translation(self, text, from_language_key, to_language_key):
        params = {
            'auth_key': self.api_key,
            'text': text,
            'source_lang': from_language_key,
            'target_lang': to_language_key
        }
        response = requests.get(self.base_url, params=params, timeout=cloudlanguagetools.constants.RequestTimeout)

        if response.status_code == 200:
            # {'translations': [{'translation': 'Le coût est très bas.'}], 'word_count': 2, 'character_count': 4}
            data = response.json()
            return data['translations'][0]['text']

        error_message = error_message = f'DeepL: could not translate text [{text}] from {from_language_key} to {to_language_key} (status_code: {response.status_code} {response.content})'
        raise cloudlanguagetools.errors.RequestError(error_message)


import enum

class Language(enum.Enum):
    af = ("Afrikaans")
    ak = ("Akan")
    ay = ("Aymara")
    bh = ("Bihari")
    sq = ("Albanian")
    am = ("Amharic")
    an = ("Aragonese")
    ar = ("Arabic")
    hy = ("Armenian")
    az = ("Azerbaijani")
    eu = ("Basque")
    ba = ("Bashkir")
    bo = ("Tibetan")
    be = ("Belarusian")
    bm = ("Bambara")
    bho = ("Bhojpuri")
    bn = ("Bangla")
    bs = ("Bosnian")
    bg = ("Bulgarian")
    ca = ("Catalan")
    ceb = ("Cebuano")
    ckb = ("Kurdish (Sorani)")
    doi = ("Dogri")
    ny = ("Chichewa")
    co = ("Corsican")
    hr = ("Croatian")
    cs = ("Czech")
    da = ("Danish")
    dv = ("Divehi")
    nl = ("Dutch")
    ee = ("Ewe")
    en = ("English")
    eo = ("Esperanto")
    et = ("Estonian")
    fo = ("Faroese")
    tl = ("Filipino")
    fi = ("Finnish")
    fr = ("French")
    fy = ("Frisian")
    gl = ("Galician")
    ka = ("Georgian")
    de = ("German")
    el = ("Greek")
    gom = ("Goan Konkani")
    gn = ("Guarani")
    gu = ("Gujarati")
    ht = ("Haitian Creole")
    ha = ("Hausa")
    haw = ("Hawaiian")
    hsb = ("Upper Sorbian")
    ikt = ("Inuinnaqtun")
    iw = ("Hebrew")
    iu = ("Inuktitut")
    iu_latn = ("Inuktitut (Latin)")
    hi = ("Hindi")
    hmn = ("Hmong")
    hu = ("Hungarian")
    is_ = ("Icelandic")
    ig = ("Igbo")
    id_ = ("Indonesian")
    ilo = ("Iloko")
    ga = ("Irish")
    it = ("Italian")
    ja = ("Japanese")
    jw = ("Javanese")
    kn = ("Kannada")
    kk = ("Kazakh")
    km = ("Khmer")
    kri = ("Krio")
    lg = ("Luganda")
    ln = ("Lingala")
    lus = ("Mizo")
    rw = ("Kinyarwanda")
    ko = ("Korean")
    ku = ("Kurdish (Central)")
    ky = ("Kyrgyz")
    lo = ("Lao")
    la = ("Latin")
    lv = ("Latvian")
    lt = ("Lithuanian")
    lb = ("Luxembourgish")
    mai = ("Maithili")
    mk = ("Macedonian")
    mg = ("Malagasy")
    ms = ("Malay")
    ml = ("Malayalam")
    mt = ("Maltese")
    mi = ("Maori")
    mr = ("Marathi")
    mn = ("Mongolian")
    mni = ("Manipuri (Meitei Mayek)")
    mn_cyrl = ("Mongolian (Cyrillic)")
    cnr = ("Montenegrin")
    my = ("Myanmar (Burmese)")
    ne = ("Nepali")
    no = ("Norwegian")
    nso = ("Northern Sotho")
    om = ("Oromo")
    or_ = ("Odia")
    ps = ("Pashto")
    qu = ("Quechua")
    fa = ("Persian")
    pl = ("Polish")
    pt_br = ("Portuguese (Brazil)")
    pa = ("Punjabi")
    ro = ("Romanian")
    ru = ("Russian")
    sm = ("Samoan")
    gd = ("Scots Gaelic")
    st = ("Sesotho")
    sn = ("Shona")
    sd = ("Sindhi")
    si = ("Sinhala")
    sk = ("Slovak")
    sl = ("Slovenian")
    so = ("Somali")
    es = ("Spanish")
    es_mx = ("Spanish (Mexico)")
    sa = ("Sanskrit")
    su = ("Sundanese")
    sw = ("Swahili")
    sv = ("Swedish")
    tg = ("Tajik")
    ta = ("Tamil")
    tt = ("Tatar")
    te = ("Telugu")
    th = ("Thai")
    ti = ("Tigrinya")
    tr = ("Turkish")
    ts = ("Tsonga")
    tk = ("Turkmen")
    uk = ("Ukrainian")
    ur = ("Urdu")
    ug = ("Uyghur")
    uz = ("Uzbek")
    vi = ("Vietnamese")
    cy = ("Welsh")
    xh = ("Xhosa")
    yi = ("Yiddish")
    yo = ("Yoruba")
    zu = ("Zulu")
    he = ("Hebrew")
    as_ = ("Assamese")
    fil = ("Filipino")
    fj = ("Fijian")
    fr_ca = ("French (Canada)")
    kmr = ("Kurdish (Northern)")
    mww = ("Hmong Daw")
    nb = ("Norwegian")
    otq = ("Querétaro Otomi")
    prs = ("Dari")
    pt_pt = ("Portuguese (Portugal)")
    sr_cyrl = ("Serbian (Cyrillic)")
    sr_latn = ("Serbian (Latin)")
    tlh_latn = ("Klingon (Latin)")
    tlh_piqd = ("Klingon (pIqaD)")
    to = ("Tongan")
    ty = ("Tahitian")
    yua = ("Yucatec Maya")
    yue = ("Chinese (Cantonese, Traditional)")
    zh_cn = ("Chinese (Simplified)")
    zh_tw = ("Chinese (Traditional)")
    zh_lit = ("Chinese (Literary)")


    def __init__(self, lang_name):
        self.lang_name = lang_name        

class AudioLanguage(enum.Enum):
    af_ZA = (Language.af, "Afrikaans (South Africa)")
    am_ET = (Language.am, "Amharic (Ethiopia)")
    an_ES = (Language.an, "Aragonese (Spain)")
    # arabic
    ar_AE = (Language.ar, "Arabic (United Arab Emirates)")
    ar_BH = (Language.ar, "Arabic (Bahrain)")
    ar_DZ = (Language.ar, "Arabic (Algeria)")
    ar_EG = (Language.ar, "Arabic (Egypt)")
    ar_IQ = (Language.ar, "Arabic (Iraq)")
    ar_JO = (Language.ar, "Arabic (Jordan)")
    ar_KW = (Language.ar, "Arabic (Kuwait)")
    ar_LB = (Language.ar, "Arabic (Lebanon)")
    ar_LY = (Language.ar, "Arabic (Libya)")
    ar_MA = (Language.ar, "Arabic (Morocco)")
    ar_OM = (Language.ar, "Arabic (Oman)")
    ar_QA = (Language.ar, "Arabic (Qatar)")
    ar_SA = (Language.ar, "Arabic (Saudi Arabia)")
    ar_SY = (Language.ar, "Arabic (Syria)")
    ar_TN = (Language.ar, "Arabic (Tunisia)")
    ar_XA = (Language.ar, "Arabic")
    ar_YE = (Language.ar, "Arabic (Yemen)")
    
    as_IN = (Language.as_, "Assamese (India)")
    az_AZ = (Language.az, "Azerbaijani (Azerbaijan)")
    ba_RU = (Language.ba, "Bashkir (Russia)")

    bg_BG = (Language.bg, "Bulgarian")
    bn_BD = (Language.bn, "Bangla (Bangladesh)")
    bn_IN = (Language.bn, "Bengali (India)")
    bs_BA = (Language.bs, "Bosnian (Bosnia and Herzegovina)")
    ca_ES = (Language.ca, "Catalan")
    cs_CZ = (Language.cs, "Czech")
    cy_GB = (Language.cy, "Welsh")
    da_DK = (Language.da, "Danish")
    fo_FO = (Language.fo, "Faroese (Faroe Islands)")
    de_AT = (Language.de, "German (Austria)")
    de_CH = (Language.de, "German (Switzerland)")
    de_DE = (Language.de, "German (Germany)")
    el_GR = (Language.el, "Greek")
    en_AU = (Language.en, "English (Australia)")
    en_CA = (Language.en, "English (Canada)")
    en_CB = (Language.en, "English (Carribean)")
    en_GB = (Language.en, "English (UK)")
    en_GB_WLS = (Language.en, "English (Welsh)")
    en_IE = (Language.en, "English (Ireland)")
    en_HK = (Language.en, "English (Hong Kong)")
    en_IN = (Language.en, "English (India)")
    en_KE = (Language.en, "English (Kenya)")
    en_NG = (Language.en, "English (Nigeria)")
    en_US = (Language.en, "English (US)")
    en_NZ = (Language.en, "English (New Zealand)")
    en_PH = (Language.en, "English (Philippines)")
    en_SG = (Language.en, "English (Singapore)")
    en_TZ = (Language.en, "English (Tanzania)")
    en_ZA = (Language.en, "English (South Africa)")
    eo_XX = (Language.eo, "Esperanto")
    # spanish
    es_AR = (Language.es, "Spanish (Argentina)")
    es_BO = (Language.es, "Spanish (Bolivia)")
    es_CL = (Language.es, "Spanish (Chile)")
    es_CO = (Language.es, "Spanish (Colombia)")
    es_CR = (Language.es, "Spanish (Costa Rica)")
    es_CU = (Language.es, "Spanish (Cuba)")
    es_DO = (Language.es, "Spanish (Dominican Republic)")
    es_EC = (Language.es, "Spanish (Ecuador)")
    es_ES = (Language.es, "Spanish (Spain)")
    es_GQ = (Language.es, "Spanish (Equatorial Guinea)")
    es_GT = (Language.es, "Spanish (Guatemala)")
    es_HN = (Language.es, "Spanish (Honduras)")
    es_LA = (Language.es, "Spanish (Latin America)")
    es_MX = (Language.es, "Spanish (Mexico)")
    es_NI = (Language.es, "Spanish (Nicaragua)")
    es_PA = (Language.es, "Spanish (Panama)")
    es_PE = (Language.es, "Spanish (Peru)")
    es_PR = (Language.es, "Spanish (Puerto Rico)")
    es_PY = (Language.es, "Spanish (Paraguay)")
    es_SV = (Language.es, "Spanish (El Salvador)")
    es_US = (Language.es, "Spanish (North America)")
    es_UY = (Language.es, "Spanish (Uruguay)")
    es_VE = (Language.es, "Spanish (Venezuela)")

    et_EE = (Language.et, "Estonian")
    eu = (Language.ba, "Basque (Basque Country)")
    fi_FI = (Language.fi, "Finnish")
    fil_PH = (Language.tl, "Filipino (Philippines)")
    fr_BE = (Language.fr, "French (Belgium)")
    fr_CA = (Language.fr_ca, "French (Canada)")
    fr_CH = (Language.fr, "French (Switzerland)")
    fr_FR = (Language.fr, "French (France)")
    ga_IE = (Language.ga, "Irish (Ireland)")
    gl_ES = (Language.gl, "Galician (Spain)")
    gd_GB = (Language.gd, "Gaelic (UK)")
    gu_IN = (Language.gu, "Gujarati (India)")
    he_IL = (Language.he, "Hebrew (Israel)")
    hi_IN = (Language.hi, "Hindi (India)")
    hr_HR = (Language.hr, "Croatian")
    hu_HU = (Language.hu, "Hungarian")
    hy_AM = (Language.hy, "Armenian (Armenia)")
    id_ID = (Language.id_, "Indonesian")
    is_IS = (Language.is_, "Icelandic")
    it_IT = (Language.it, "Italian")
    ja_JP = (Language.ja, "Japanese")
    jv_ID = (Language.jw, "Javanese (Indonesia)")
    ka_GE = (Language.ka, "Georgian (Georgia)")
    ko_KR = (Language.ko, "Korean")
    kk_KZ = (Language.kk, "Kazakh (Kazakhstan)")
    kn_IN = (Language.kn, "Kannada (India)")
    km_KH = (Language.km, "Khmer (Cambodia)")
    la = (Language.la, "Latin")
    lo_LA = (Language.lo, "Lao (Lao People's Democratic Republic")
    lt_LT = (Language.lt, "Lithuanian")
    lv_LV = (Language.lv, "Latvian")
    mk_MK = (Language.mk, "Macedonian (Macedonia)")
    mn_MN = (Language.mn, "Mongolian (Mongolia)")
    ml_IN = (Language.ml, "Malayalam (India)")
    mr_IN = (Language.mr, "Marathi (India)")
    ms_MY = (Language.ms, "Malay")
    mt_MT = (Language.mt, "Maltese (Malta)")
    my_MM = (Language.my, "Burmese (Myanmar [Burma])")
    nb_NO = (Language.nb, "Norwegian")
    ne_NP = (Language.ne, "Nepali (Nepal)")
    nl_BE = (Language.nl, "Dutch (Belgium)")
    nl_NL = (Language.nl, "Dutch")
    pa_IN = (Language.pa, "Punjabi (India)")
    pl_PL = (Language.pl, "Polish")
    ps_AF = (Language.ps, "Pashto (Afghanistan)")
    pt_BR = (Language.pt_br, "Portuguese (Brazil)")
    pt_PT = (Language.pt_pt, "Portuguese (Portugal)")
    ro_RO = (Language.ro, "Romanian")
    ru_RU = (Language.ru, "Russian")
    si_LK = (Language.si, "Sinhala (Sri Lanka)")
    sk_SK = (Language.sk, "Slovak")
    sl_SI = (Language.sl, "Slovenian")
    so_SO = (Language.so, "Somali (Somalia)")
    sq_AL = (Language.sq, "Albanian (Albania)")
    sr_RS = (Language.sr_cyrl, "Serbian (Serbia)")
    su_ID = (Language.su, "Sundanese (Indonesia)")
    sv_SE = (Language.sv, "Swedish")
    sw_KE = (Language.sw, "Swahili (Kenya)")
    sw_TZ = (Language.sw, "Swahili (Tanzania)")
    ta_MY = (Language.ta, "Tamil (Malaysia)")
    ta_IN = (Language.ta, "Tamil (India)")
    ta_LK = (Language.ta, "Tamil (Sri Lanka)")
    ta_SG = (Language.ta, "Tamil (Singapore)")
    te_IN = (Language.te, "Telugu (India)")
    th_TH = (Language.th, "Thai")
    tr_TR = (Language.tr, "Turkish (Turkey)")
    tt_RU = (Language.tt, "Tatar (Russia)")
    ur_IN = (Language.ur, "Urdu (India)")
    ur_PK = (Language.ur, "Urdu (Pakistan)")
    uk_UA = (Language.uk, "Ukrainian (Ukraine)")
    uz_UZ = (Language.uz, "Uzbek (Uzbekistan)")
    vi_VN = (Language.vi, "Vietnamese")
    zh_CN = (Language.zh_cn, "Chinese (Mandarin, Simplified)")
    zh_HK = (Language.yue, "Chinese (Cantonese, Traditional)")
    zh_TW = (Language.zh_tw, "Chinese (Taiwanese Mandarin)")
    fa_IR = (Language.fa, "Persian (Iran)")
    zu_ZA = (Language.zu, "Zulu (South Africa)")

    def __init__(self, lang, audio_lang_name):
        self.lang = lang
        self.audio_lang_name = audio_lang_name    


AudioLanguageDefaults = {
    Language.ar: AudioLanguage.ar_XA,
    Language.bn: AudioLanguage.bn_BD,
    Language.de: AudioLanguage.de_DE,
    Language.en: AudioLanguage.en_US,
    Language.es: AudioLanguage.es_ES,
    Language.fr: AudioLanguage.fr_FR,
    Language.nl: AudioLanguage.nl_NL,
    Language.sw: AudioLanguage.sw_TZ,
    Language.ta: AudioLanguage.ta_IN,
    Language.ur: AudioLanguage.ur_PK
}

# if only one locale is available, make that the default
language_map_to_audio_language = {}
for audio_language in AudioLanguage:
    if audio_language.lang not in language_map_to_audio_language:
        language_map_to_audio_language[audio_language.lang] = []
    language_map_to_audio_language[audio_language.lang].append(audio_language)
for key, value in language_map_to_audio_language.items():
    if len(value) == 1:
        AudioLanguageDefaults[key] = value[0]




import json
import requests
import tempfile
import uuid
import operator
import pydub
import logging

import cloudlanguagetools.service
import cloudlanguagetools.constants
import cloudlanguagetools.options
import cloudlanguagetools.languages
import cloudlanguagetools.ttsvoice
import cloudlanguagetools.translationlanguage
import cloudlanguagetools.transliterationlanguage
import cloudlanguagetools.errors


import azure.cognitiveservices.speech
import azure.cognitiveservices.speech.audio

AUDIO_LOCALE_OVERRIDE_MAP = {
    'zh-CN-liaoning': 'zh_CN',
    'zh-CN-sichuan': 'zh_CN',
}

class AzureVoice(cloudlanguagetools.ttsvoice.TtsVoice):
    def __init__(self, voice_data):
        # print(voice_data)
        self.service = cloudlanguagetools.constants.Service.Azure
        locale = voice_data['Locale']
        locale = AUDIO_LOCALE_OVERRIDE_MAP.get(locale, locale)
        language_enum_name = locale.replace('-', '_')
        self.audio_language = cloudlanguagetools.languages.AudioLanguage[language_enum_name]
        self.name = voice_data['Name']
        self.display_name = voice_data['DisplayName']
        self.local_name = voice_data['LocalName']
        self.short_name = voice_data['ShortName']
        self.gender = cloudlanguagetools.constants.Gender[voice_data['Gender']]

        self.locale = locale
        self.voice_type = voice_data['VoiceType']


    def get_voice_key(self):
        return {
            'name': self.name
        }

    def get_voice_shortname(self):
        if self.local_name != self.display_name:
            return f'{self.display_name} {self.local_name} ({self.voice_type})'
        else:
            return f'{self.display_name} ({self.voice_type})'

    def get_options(self):
        return {
            'rate' : {
                'type': cloudlanguagetools.options.ParameterType.number.name,
                'min': 0.5,
                'max': 3.0,
                'default': 1.0
            },
            'pitch': {
                'type': cloudlanguagetools.options.ParameterType.number.name,
                'min': -100,
                'max': 100,
                'default': 0
            },
            cloudlanguagetools.options.AUDIO_FORMAT_PARAMETER: {
                'type': cloudlanguagetools.options.ParameterType.list.name,
                'values': [
                    cloudlanguagetools.options.AudioFormat.mp3.name,
                    cloudlanguagetools.options.AudioFormat.ogg_opus.name,
                ],
                'default': cloudlanguagetools.options.AudioFormat.mp3.name
            }
        }

def get_translation_language_enum(language_id):
    # print(f'language_id: {language_id}')
    azure_language_id_map = {
        'as': 'as_',
        'fr-ca': 'fr_ca',
        'fr-CA': 'fr_ca',
        'id': 'id_',
        'is': 'is_',
        'or': 'or_',
        'pt': 'pt_br',
        'pt-pt': 'pt_pt',
        'pt-PT': 'pt_pt',
        'sr-Cyrl': 'sr_cyrl',
        'sr-Latn': 'sr_latn',
        'tlh-Latn': 'tlh_latn',
        'tlh-Piqd': 'tlh_piqd',
        'zh-Hans': 'zh_cn',
        'zh-Hant': 'zh_tw',
        'lzh': 'zh_lit',
        'mn-Cyrl': 'mn_cyrl',
        'mn-Mong': 'mn',
        'iu-Latn': 'iu_latn'
    }
    if language_id in azure_language_id_map:
        language_id = azure_language_id_map[language_id]
    return cloudlanguagetools.languages.Language[language_id]

class AzureTranslationLanguage(cloudlanguagetools.translationlanguage.TranslationLanguage):
    def __init__(self, language_id):
        self.service = cloudlanguagetools.constants.Service.Azure
        self.language_id = language_id
        self.language = get_translation_language_enum(language_id)

    def get_language_id(self):
        return self.language_id

class AzureTransliterationLanguage(cloudlanguagetools.transliterationlanguage.TransliterationLanguage):
    def __init__(self, language_id, from_script, to_script, from_script_name, to_script_name):
        self.service = cloudlanguagetools.constants.Service.Azure
        self.language_id = language_id
        self.language = get_translation_language_enum(language_id)
        self.from_script = from_script
        self.to_script = to_script
        self.from_script_name = from_script_name
        self.to_script_name = to_script_name

    def get_transliteration_name(self):
        result = f'{self.language.lang_name} ({self.from_script_name} to {self.to_script_name}), {self.service.name}'
        return result

    def get_transliteration_shortname(self):
        result = f'{self.from_script_name} to {self.to_script_name}, {self.service.name}'
        return result        

    def get_transliteration_key(self):
        return {
            'language_id': self.language_id,
            'from_script': self.from_script,
            'to_script': self.to_script
        }


class AzureService(cloudlanguagetools.service.Service):
    def __init__(self):
        self.url_translator_base = 'https://api.cognitive.microsofttranslator.com'

    def configure(self, config):
        self.key = config['key']
        self.region = config['region']

    def get_token(self):
        fetch_token_url = f"https://{self.region}.api.cognitive.microsoft.com/sts/v1.0/issueToken"
        headers = {
            'Ocp-Apim-Subscription-Key': self.key
        }
        response = requests.post(fetch_token_url, headers=headers, timeout=cloudlanguagetools.constants.RequestTimeout)
        access_token = str(response.text)
        return access_token

    def get_translator_headers(self):
        headers = {
            'Ocp-Apim-Subscription-Key': self.key,
            'Ocp-Apim-Subscription-Region': self.region,
            'Content-type': 'application/json',
            'X-ClientTraceId': str(uuid.uuid4())
        }
        return headers        

    def get_tts_audio(self, text, voice_key, options):

        audio_format_str = options.get(cloudlanguagetools.options.AUDIO_FORMAT_PARAMETER, cloudlanguagetools.options.AudioFormat.mp3.name)
        audio_format = cloudlanguagetools.options.AudioFormat[audio_format_str]

        audio_format_map = {
            cloudlanguagetools.options.AudioFormat.mp3: 'Audio24Khz96KBitRateMonoMp3',
            cloudlanguagetools.options.AudioFormat.ogg_opus: 'Ogg48Khz16BitMonoOpus'
        }

        output_temp_file = tempfile.NamedTemporaryFile()
        output_temp_filename = output_temp_file.name
        speech_config = azure.cognitiveservices.speech.SpeechConfig(subscription=self.key, region=self.region)
        speech_config.set_speech_synthesis_output_format(azure.cognitiveservices.speech.SpeechSynthesisOutputFormat[audio_format_map[audio_format]])
        synthesizer = azure.cognitiveservices.speech.SpeechSynthesizer(speech_config=speech_config, audio_config=None)

        default_pitch = 0
        default_rate = 1.0

        pitch = options.get('pitch', default_pitch)
        pitch_str = f'{pitch:+.0f}Hz'
        rate = options.get('rate', default_rate)
        rate_str = f'{rate:0.1f}'


        prosody_start_str = ''
        prosody_end_str = ''

        if pitch != default_pitch or rate != default_rate:
            prosody_start_str = f"""<prosody pitch="{pitch_str}" rate="{rate_str}" >"""
            prosody_end_str = """</prosody>"""

        # do some cleaning on the text
        text = text.replace(' & ', ' &amp; ')

        # the ssml str must be super optimized to have no whitespace, no extra characters
        ssml_str = f"""<speak version="1.0" xmlns="https://www.w3.org/2001/10/synthesis" xmlns:mstts="https://www.w3.org/2001/mstts" xml:lang="en-US">
<voice name="{voice_key['name']}">
{prosody_start_str}""".replace('\n', '') + text + f"""
{prosody_end_str}
</voice>
</speak>""".replace('\n', '')

        # print(f'[{ssml_str}] len: {len(ssml_str)}')

        result = synthesizer.speak_ssml(ssml_str)
        if result.reason != azure.cognitiveservices.speech.ResultReason.SynthesizingAudioCompleted:
            error_message = f'Could not generate audio: {result.cancellation_details}'
            raise cloudlanguagetools.errors.RequestError(error_message)

        stream = azure.cognitiveservices.speech.AudioDataStream(result)
        stream.save_to_wav_file(output_temp_filename)

        return output_temp_file

    def get_tts_voice_list(self):
        # returns list of TtSVoice

        token = self.get_token()

        base_url = f'https://{self.region}.tts.speech.microsoft.com/'
        path = 'cognitiveservices/voices/list'
        constructed_url = base_url + path
        headers = {
            'Authorization': 'Bearer ' + token,
        }        
        response = requests.get(constructed_url, headers=headers)
        if response.status_code == 200:
            voice_list = json.loads(response.content)
            result = []
            for voice_data in voice_list:
                # print(voice_data)
                try:
                    result.append(AzureVoice(voice_data))
                except KeyError:
                    logging.error(f'could not process voice for {voice_data}', exc_info=True)
            return result

    def get_translation(self, text, from_language_key, to_language_key):
        base_url = f'{self.url_translator_base}/translate?api-version=3.0'
        params = f'&to={to_language_key}&from={from_language_key}'
        url = base_url + params

        # You can pass more than one object in body.
        body = [{
            'text': text
        }]
        request = requests.post(url, headers=self.get_translator_headers(), json=body, timeout=cloudlanguagetools.constants.RequestTimeout)
        response = request.json()

        if 'error' in response:
            error_message = f'Azure: could not translate text [{text}] from {from_language_key} to {to_language_key} ({response})'
            raise cloudlanguagetools.errors.RequestError(error_message)

        return response[0]['translations'][0]['text']

    def get_transliteration(self, text, transliteration_key):
        return self.transliteration(text, transliteration_key['language_id'], transliteration_key['from_script'], transliteration_key['to_script'])

    def get_translation_language_list(self):
        azure_data = self.get_supported_languages()
        result = []
        for language_id, data in azure_data['translation'].items():
            try:
                result.append(AzureTranslationLanguage(language_id))
            except KeyError:
                logging.error(f'could not process translation language for {language_id}, {data}', exc_info=True)
        return result

    def get_transliteration_language_list(self):
        result = []
        azure_data = self.get_supported_languages()
        for language_id, data in azure_data['transliteration'].items():
            # get the first script
            first_script = data['scripts'][0]
            from_script =  first_script['code']
            to_script = first_script['toScripts'][0]['code']
            from_script_name = first_script['name']
            to_script_name = first_script['toScripts'][0]['name']
            # print(language_id, from_script, to_script)
            # assert(to_script == 'Latn')
            try:
                result.append(AzureTransliterationLanguage(language_id, from_script, to_script, from_script_name, to_script_name))
            except KeyError:
                logging.error(f'could not process transliteration language for {language_id}, {data}', exc_info=True)
        return result


    def get_supported_languages(self):
        url = 'https://api.cognitive.microsofttranslator.com/languages?api-version=3.0'

        # If you encounter any issues with the base_url or path, make sure
        # that you are using the latest endpoint: https://docs.microsoft.com/azure/cognitive-services/translator/reference/v3-0-languages
        path = '/languages?api-version=3.0'

        headers = {
            'Content-type': 'application/json',
            'X-ClientTraceId': str(uuid.uuid4())
        }

        request = requests.get(url, headers=headers)
        response = request.json()

        return response

        # print(json.dumps(response, sort_keys=True, indent=4, ensure_ascii=False, separators=(',', ': ')))        


    def detect_language(self, text_list):
        url = f'{self.url_translator_base}/detect?api-version=3.0'
        body = [{'text': text} for text in text_list]

        request = requests.post(url, headers=self.get_translator_headers(), json=body, timeout=cloudlanguagetools.constants.RequestTimeout)
        response = request.json()

        language_score = {}
        for entry in response:
            score = entry['score']
            language = entry['language']
            if language not in language_score:
                language_score[language] = 0
            language_score[language] += score

        highest_language = max(language_score.items(), key=operator.itemgetter(1))[0]
        return get_translation_language_enum(highest_language)

    def transliteration(self, text, language_key, from_script, to_script):
        url = f'{self.url_translator_base}/transliterate?api-version=3.0'
        params = f'&language={language_key}&fromScript={from_script}&toScript={to_script}'
        constructed_url = url + params

        body = [{
            'text': text
        }]
        request = requests.post(constructed_url, headers=self.get_translator_headers(), json=body, timeout=cloudlanguagetools.constants.RequestTimeout)
        response = request.json()

        assert(len(response) == 1)
        return response[0]['text']

    # supported languages: https://docs.microsoft.com/en-us/azure/cognitive-services/speech-service/language-support#speech-to-text
    def speech_to_text(self, mp3_filepath, language, audio_format):
        speech_config = azure.cognitiveservices.speech.SpeechConfig(subscription=self.key, region=self.region)

        if audio_format == cloudlanguagetools.options.AudioFormat.mp3:
            sound = pydub.AudioSegment.from_mp3(mp3_filepath)
        elif audio_format in [cloudlanguagetools.options.AudioFormat.ogg_opus, cloudlanguagetools.options.AudioFormat.ogg_vorbis]:
            sound = pydub.AudioSegment.from_ogg(mp3_filepath)
        wav_filepath = tempfile.NamedTemporaryFile(suffix='.wav').name
        sound.export(wav_filepath, format="wav")

        audio_input = azure.cognitiveservices.speech.audio.AudioConfig(filename=wav_filepath)

        # Creates a recognizer with the given settings
        speech_recognizer = azure.cognitiveservices.speech.SpeechRecognizer(speech_config=speech_config, audio_config=audio_input, language=language)

        # Starts speech recognition, and returns after a single utterance is recognized. The end of a
        # single utterance is determined by listening for silence at the end or until a maximum of 15
        # seconds of audio is processed.  The task returns the recognition text as result. 
        # Note: Since recognize_once() returns only a single utterance, it is suitable only for single
        # shot recognition like command or query. 
        # For long-running multi-utterance recognition, use start_continuous_recognition() instead.
        result = speech_recognizer.recognize_once()

        # Checks result.
        if result.reason == azure.cognitiveservices.speech.ResultReason.RecognizedSpeech:
            return result.text
        elif result.reason == azure.cognitiveservices.speech.ResultReason.NoMatch:
            error_message = "No speech could be recognized: {}".format(result.no_match_details)
            raise Exception(error_message)
        elif result.reason == azure.cognitiveservices.speech.ResultReason.Canceled:
            cancellation_details = result.cancellation_details
            error_message = "Speech Recognition canceled: {}".format(cancellation_details)
            raise Exception(error_message)

        raise "unknown error"

    def dictionary_lookup(self, input_text, from_language_key, to_language_key):
        base_url = f'{self.url_translator_base}/dictionary/lookup?api-version=3.0'
        params = f'&to={to_language_key}&from={from_language_key}'
        url = base_url + params

        logging.info(f'querying url for dictionary lookup: {url}')

        # You can pass more than one object in body.
        body = [{
            'text': input_text
        }]
        request = requests.post(url, headers=self.get_translator_headers(), json=body, timeout=cloudlanguagetools.constants.RequestTimeout)
        response = request.json()

        translation_entries = response[0]['translations']
        translations = [entry['displayTarget'] for entry in translation_entries]

        return translations
        
        # print(json.dumps(response, sort_keys=True, indent=4, ensure_ascii=False, separators=(',', ': ')))

        # return response[0]['translations'][0]['text']

    
    def dictionary_examples(self, input_text, translation, from_language_key, to_language_key):
        base_url = f'{self.url_translator_base}/dictionary/examples?api-version=3.0'
        params = f'&to={to_language_key}&from={from_language_key}'
        url = base_url + params

        print(url)

        # You can pass more than one object in body.
        body = [{
            'text': input_text,
            'translation': translation
        }]
        request = requests.post(url, headers=self.get_translator_headers(), json=body, timeout=cloudlanguagetools.constants.RequestTimeout)
        response = request.json()

        print(json.dumps(response, sort_keys=True, indent=4, ensure_ascii=False, separators=(',', ': ')))

        # return response[0]['translations'][0]['text']    

    


import os
import tempfile
import html
import base64
import logging
import google.cloud.texttospeech
import google.cloud.translate_v2
import google.api_core.exceptions
import cloudlanguagetools.service
import cloudlanguagetools.constants
import cloudlanguagetools.options
import cloudlanguagetools.languages

def language_code_to_enum(language_code):
    override_map = {
        'cmn-TW': cloudlanguagetools.languages.AudioLanguage.zh_TW,
        'cmn-CN': cloudlanguagetools.languages.AudioLanguage.zh_CN,
        'yue-HK': cloudlanguagetools.languages.AudioLanguage.zh_HK
    }
    if language_code in override_map:
        return override_map[language_code]
    language_enum_name = language_code.replace('-', '_')
    return cloudlanguagetools.languages.AudioLanguage[language_enum_name]

class GoogleVoice(cloudlanguagetools.ttsvoice.TtsVoice):
    def __init__(self, voice_data):
        self.service = cloudlanguagetools.constants.Service.Google
        self.name = voice_data.name
        self.google_ssml_gender = google.cloud.texttospeech.SsmlVoiceGender(voice_data.ssml_gender)
        gender_str = self.google_ssml_gender.name.lower().capitalize()
        self.gender = cloudlanguagetools.constants.Gender[gender_str]
        assert len(voice_data.language_codes) == 1
        self.google_language_code = voice_data.language_codes[0]
        self.audio_language = language_code_to_enum(self.google_language_code)

    def get_voice_shortname(self):
        return self.name

    def get_voice_key(self):
        return {
            'name': self.name,
            'language_code': self.google_language_code,
            'ssml_gender': self.google_ssml_gender.name
        }

    def get_options(self):
        return {
            'speaking_rate': {
                'type': 'number',
                'min': 0.25,
                'max': 4.0,
                'default': 1.0
            },
            'pitch': {
                'type': 'number',
                'min': -20.0,
                'max': 20.0,
                'default': 0.0
            },
            cloudlanguagetools.options.AUDIO_FORMAT_PARAMETER: {
                'type': cloudlanguagetools.options.ParameterType.list.name,
                'values': [
                    cloudlanguagetools.options.AudioFormat.mp3.name,
                    cloudlanguagetools.options.AudioFormat.ogg_opus.name,
                ],
                'default': cloudlanguagetools.options.AudioFormat.mp3.name
            }            
        }        


def get_translation_language_enum(language_id):
    google_language_id_map = {
        'as': 'as_',
        'fr-ca': 'fr_ca',
        'id': 'id_',
        'is': 'is_',
        'or': 'or_',
        'pt': 'pt_br',
        'pt-pt': 'pt_pt',
        'sr': 'sr_cyrl',
        'sr-Cyrl': 'sr_cyrl',
        'sr-Latn': 'sr_latn',
        'tlh-Latn': 'tlh_latn',
        'tlh-Piqd': 'tlh_piqd',
        'zh-CN': 'zh_cn',
        'zh-TW': 'zh_tw',
        'zh': 'zh_cn',
        'mni-Mtei': 'mni',
    }
    if language_id in google_language_id_map:
        language_id = google_language_id_map[language_id]
    return cloudlanguagetools.languages.Language[language_id]

class GoogleTranslationLanguage(cloudlanguagetools.translationlanguage.TranslationLanguage):
    def __init__(self, language_id):
        self.service = cloudlanguagetools.constants.Service.Google
        self.language_id = language_id
        self.language = get_translation_language_enum(language_id)

    def get_language_id(self):
        return self.language_id


class GoogleService(cloudlanguagetools.service.Service):
    def __init__(self):
        pass

    def configure(self, config):
        data_bytes = base64.b64decode(config['key'])
        data_str = data_bytes.decode('utf-8')    
        # write to file
        # note: temp file needs to be a member so it doesn't get collected
        self.google_key_temp_file = tempfile.NamedTemporaryFile()  
        google_key_filename = self.google_key_temp_file.name
        with open(google_key_filename, 'w') as f:
            f.write(data_str)    
            f.close()
        os.environ['GOOGLE_APPLICATION_CREDENTIALS'] = google_key_filename

    def get_client(self):
        client = google.cloud.texttospeech.TextToSpeechClient()
        return client

    def get_translation_client(self):
        translate_client = google.cloud.translate_v2.Client()
        return translate_client

    def get_tts_audio(self, text, voice_key, options):
        audio_format_str = options.get(cloudlanguagetools.options.AUDIO_FORMAT_PARAMETER, cloudlanguagetools.options.AudioFormat.mp3.name)
        audio_format = cloudlanguagetools.options.AudioFormat[audio_format_str]

        audio_format_map = {
            cloudlanguagetools.options.AudioFormat.mp3: google.cloud.texttospeech.AudioEncoding.MP3,
            cloudlanguagetools.options.AudioFormat.ogg_opus: google.cloud.texttospeech.AudioEncoding.OGG_OPUS
        }

        client = self.get_client()

        ssml_text = '<speak>' + text + '</speak>'
        input_text = google.cloud.texttospeech.SynthesisInput(ssml=ssml_text)

        # Note: the voice can also be specified by name.
        # Names of voices can be retrieved with client.list_voices().
        voice = google.cloud.texttospeech.VoiceSelectionParams(
            name=voice_key['name'],
            language_code=voice_key['language_code'],
            ssml_gender=google.cloud.texttospeech.SsmlVoiceGender[voice_key['ssml_gender']]
        )

        audio_config = google.cloud.texttospeech.AudioConfig(
            audio_encoding=audio_format_map[audio_format],
            speaking_rate=options.get('speaking_rate', 1.0),
            pitch=options.get('pitch', 0.0)
        )

        response = client.synthesize_speech(
            request={"input": input_text, "voice": voice, "audio_config": audio_config}
        )

        # The response's audio_content is binary.
        output_temp_file = tempfile.NamedTemporaryFile()
        output_temp_filename = output_temp_file.name        
        with open(output_temp_filename, "wb") as out:
            out.write(response.audio_content)

        return output_temp_file


    def get_tts_voice_list(self):
        client = self.get_client()

        # Performs the list voices request
        voices = client.list_voices()

        result = []

        for voice in voices.voices:
            try:
                result.append(GoogleVoice(voice))
            except KeyError:
                logging.error(f'could not process voice for {voice}', exc_info=True)

        return result

    def get_translation_language_list(self):
        data = self.get_translation_languages()
        result_dict = {}
        for entry in data:
            try:
                translation_language = GoogleTranslationLanguage(entry['language'])
                result_dict[translation_language.get_language_code()] = translation_language
            except KeyError:
                logging.error(f'could not process translation language for {entry}', exc_info=True)                
        return result_dict.values()

    def get_transliteration_language_list(self):
        return []

    def get_translation(self, text, from_language_key, to_language_key):
        try:
            client = self.get_translation_client()
            result = client.translate(text, source_language=from_language_key, target_language=to_language_key)
            return html.unescape(result["translatedText"])
        except google.api_core.exceptions.BadRequest as error:
            raise cloudlanguagetools.errors.RequestError(str(error))

    def get_translation_languages(self):
        translate_client = google.cloud.translate_v2.Client()

        results = translate_client.get_languages()

        return results

    def detect_language(self, input_text):
        client = self.get_translation_client()

        # Text can also be a sequence of strings, in which case this method
        # will return a sequence of results for each text.
        result = client.detect_language(input_text)

        print("Text: {}".format(input_text))
        print("Confidence: {}".format(result["confidence"]))
        print("Language: {}".format(result["language"]))


import os
import base64
import tempfile
import logging
import timeit
import cloudlanguagetools.constants
import cloudlanguagetools.languages
import cloudlanguagetools.errors
import cloudlanguagetools.azure
import cloudlanguagetools.google
import cloudlanguagetools.mandarincantonese
import cloudlanguagetools.easypronunciation
import cloudlanguagetools.watson
import cloudlanguagetools.naver
import cloudlanguagetools.amazon
import cloudlanguagetools.forvo
import cloudlanguagetools.cereproc
import cloudlanguagetools.epitran
import cloudlanguagetools.deepl
import cloudlanguagetools.vocalware
import cloudlanguagetools.fptai
import cloudlanguagetools.voicen
import cloudlanguagetools.pythainlp
import cloudlanguagetools.spacy
import cloudlanguagetools.argostranslate
import cloudlanguagetools.encryption

class ServiceManager():
    def  __init__(self):
        self.services = {}
        self.services[cloudlanguagetools.constants.Service.Azure.name] = cloudlanguagetools.azure.AzureService()
        self.services[cloudlanguagetools.constants.Service.Google.name] = cloudlanguagetools.google.GoogleService()
        self.services[cloudlanguagetools.constants.Service.EasyPronunciation.name] = cloudlanguagetools.easypronunciation.EasyPronunciationService()
        self.services[cloudlanguagetools.constants.Service.Watson.name] = cloudlanguagetools.watson.WatsonService()
        self.services[cloudlanguagetools.constants.Service.Naver.name] = cloudlanguagetools.naver.NaverService()
        self.services[cloudlanguagetools.constants.Service.Amazon.name] = cloudlanguagetools.amazon.AmazonService()
        self.services[cloudlanguagetools.constants.Service.Forvo.name] = cloudlanguagetools.forvo.ForvoService()
        self.services[cloudlanguagetools.constants.Service.CereProc.name] = cloudlanguagetools.cereproc.CereProcService()
        self.services[cloudlanguagetools.constants.Service.Epitran.name] = cloudlanguagetools.epitran.EpitranService()
        self.services[cloudlanguagetools.constants.Service.DeepL.name] = cloudlanguagetools.deepl.DeepLService()
        self.services[cloudlanguagetools.constants.Service.VocalWare.name] = cloudlanguagetools.vocalware.VocalWareService()
        self.services[cloudlanguagetools.constants.Service.FptAi.name] = cloudlanguagetools.fptai.FptAiService()
        self.services[cloudlanguagetools.constants.Service.Voicen.name] = cloudlanguagetools.voicen.VoicenService()
        self.services[cloudlanguagetools.constants.Service.PyThaiNLP.name] = cloudlanguagetools.pythainlp.PyThaiNLPService()
        self.services[cloudlanguagetools.constants.Service.Spacy.name] = cloudlanguagetools.spacy.SpacyService()
        self.services[cloudlanguagetools.constants.Service.MandarinCantonese.name] = cloudlanguagetools.mandarincantonese.MandarinCantoneseService()            
        self.services[cloudlanguagetools.constants.Service.ArgosTranslate.name] = cloudlanguagetools.argostranslate.ArgosTranslateService()

    def configure_default(self):
        # use the stored keys to configure services
        self.configure_services(cloudlanguagetools.encryption.decrypt())

    def configure_services(self, config):
        for service_name, value in config.items():
            self.services[service_name].configure(value)

    def get_language_data_json(self):
        # retrieve all language data (tts, translation, transliteration, etc)
        logging.info('retrieving language data')
        
        logging.info('retrieving language list')
        language_list = self.get_language_list()
        logging.info('retrieving translation list')
        translation_language_list = self.get_translation_language_list()
        logging.info('retrieving transliteration language list')
        transliteration_language_list = self.get_transliteration_language_list()
        logging.info('retrieving tts voice list')
        tts_voice_list = self.get_tts_voice_list()
        logging.info('retrieving tokenization options')
        tokenization_options = self.get_tokenization_options()

        return {
            'language_list': language_list,
            'translation_options': [option.json_obj() for option in translation_language_list],
            'transliteration_options': [option.json_obj() for option in transliteration_language_list],
            'voice_list': [voice.json_obj() for voice in tts_voice_list],
            'tokenization_options': [option.json_obj() for option in tokenization_options],
        }
        

    def get_language_list(self):
        result_dict = {}
        for language in cloudlanguagetools.languages.Language:
            result_dict[language.name] = language.lang_name
        return result_dict

    def get_tts_voice_list(self):
        result = []
        for key, service in self.services.items():
            logging.info(f'retrieving voice list from {key}')
            result.extend(service.get_tts_voice_list())
        return result

    def get_tts_voice_list_json(self):
        tts_voice_list = self.get_tts_voice_list()
        return [voice.json_obj() for voice in tts_voice_list]

    def get_translation_language_list(self):
        result = []
        for key, service in self.services.items():
            result.extend(service.get_translation_language_list())
        return result        

    def get_translation_language_list_json(self):
        """return list of languages supported for translation, using plain objects/strings"""
        language_list = self.get_translation_language_list()
        return [language.json_obj() for language in language_list]

    def get_transliteration_language_list(self):
        result = []
        for key, service in self.services.items():
            result.extend(service.get_transliteration_language_list())
        return result

    def get_transliteration_language_list_json(self):
        """return list of languages supported for transliteration, using plain objects/strings"""
        language_list = self.get_transliteration_language_list()
        return [language.json_obj() for language in language_list]

    def get_tokenization_options(self):
        result = []
        for key, service in self.services.items():
            result.extend(service.get_tokenization_options())
        return result

    def get_tokenization_options_json(self):
        """return list of languages supported for tokenization, using plain objects/strings"""
        tokenization_list = self.get_tokenization_options()
        return [tokenization_option.json_obj() for tokenization_option in tokenization_list]

    def get_tts_audio(self, text, service, voice_id, options):
        service = self.services[service]
        return service.get_tts_audio(text, voice_id, options)

    def get_translation(self, text, service, from_language_key, to_language_key):
        """return text"""
        service = self.services[service]
        return service.get_translation(text, from_language_key, to_language_key)

    def get_all_translations(self, text, from_language, to_language):
        global_starttime = timeit.default_timer()

        translation_language_list = self.get_translation_language_list()

        result = {}
        for service_name, service in self.services.items():
            # locate from language key
            from_language_entries = [x for x in translation_language_list if x.service.name == service_name and x.get_language_code() == from_language]
            if len(from_language_entries) == 1:
                starttime = timeit.default_timer()
                # this service provides the "from" language in translation list
                from_language_id = from_language_entries[0].get_language_id()
                # locate to language key
                to_language_entries = [x for x in translation_language_list if x.service.name == service_name and x.get_language_code() == to_language]
                if len(to_language_entries) == 1:
                    to_language_id = to_language_entries[0].get_language_id()
                    try:
                        result[service_name] = self.get_translation(text, service_name, from_language_id, to_language_id)
                    except cloudlanguagetools.errors.RequestError:
                        pass # don't do anything
                    time_diff = timeit.default_timer() - starttime
                    logging.info(f'get_all_translation processing time for {service_name}: {time_diff:.1f}')
        global_time_diff = timeit.default_timer() - global_starttime
        logging.info(f'get_all_translation total processing time: {global_time_diff:.1f}')
        return result

    def get_transliteration(self, text, service, transliteration_key):
        service = self.services[service]
        return service.get_transliteration(text, transliteration_key)

    def get_tokenization(self, text, service, tokenization_key):
        service = self.services[service]
        return service.get_tokenization(text, tokenization_key)

    def get_breakdown(self, text, tokenization_option, translation_option, transliteration_option):
        
        # first, tokenize
        tokenization_service = tokenization_option['service']
        tokenization_key = tokenization_option['tokenization_key']
        tokenization_result = self.get_tokenization(text, tokenization_service, tokenization_key)

        if translation_option != None:
            translation_service = translation_option['service']
            translation_source_language_id = translation_option['source_language_id']
            translation_target_language_id = translation_option['target_language_id']

        if transliteration_option != None:
            transliteration_service = transliteration_option['service']
            transliteration_key = transliteration_option['transliteration_key']

        # then, enrich tokens with translation and transliteration
        result = []
        for token in tokenization_result:
            entry = {
                'token': token['token'],
                'lemma': token['lemma']
            }

            if token['can_translate'] and translation_option != None:
                # translate this token (lemma)
                entry['translation'] = self.get_translation(token['lemma'], translation_service, translation_source_language_id, translation_target_language_id)

            if token['can_transliterate'] and transliteration_option != None:
                # transliterate the token
                entry['transliteration'] = self.get_transliteration(token['token'], transliteration_service, transliteration_key)

            if 'pos_description' in token:
                entry['pos_description'] = token['pos_description']

            result.append(entry)
        
        return result



    def detect_language(self, text_list):
        """returns an enum from languages.Language"""
        service = self.services[cloudlanguagetools.constants.Service.Azure.name]
        result = service.detect_language(text_list)
        return result


import pythainlp
import enum
import string

import cloudlanguagetools.service
import cloudlanguagetools.constants
import cloudlanguagetools.languages
import cloudlanguagetools.tokenization

class PyThaiNLPTransliterationMode(enum.Enum):
    Romanization = enum.auto()
    IPA = enum.auto()

class PyThaiNLPTokenizationMode(enum.Enum):
    Default = enum.auto()

class PyThaiNLPTransliterationLanguage(cloudlanguagetools.transliterationlanguage.TransliterationLanguage):
    def __init__(self, mode):
        self.language = cloudlanguagetools.languages.Language.th
        self.service = cloudlanguagetools.constants.Service.PyThaiNLP
        self.mode = mode

    def get_transliteration_name(self):
        result = f'{self.language.lang_name} ({self.mode.name}), {self.service.name}'
        return result

    def get_transliteration_shortname(self):
        result = f'{self.mode.name}, {self.service.name}'
        return result        

    def get_transliteration_key(self):
        return {
            'mode': self.mode.name
        }

class PyThaiNLPTokenization(cloudlanguagetools.tokenization.Tokenization):
    def __init__(self, mode):
        self.language = cloudlanguagetools.languages.Language.th
        self.service = cloudlanguagetools.constants.Service.PyThaiNLP
        self.mode = mode

    def get_tokenization_name(self):
        result = f'{self.language.lang_name} ({self.mode.name}), {self.service.name}'
        return result

    def get_tokenization_key(self):
        return {
            'mode': self.mode.name
        }

class PyThaiNLPService(cloudlanguagetools.service.Service):
    def __init__(self):
        pass

    def get_transliteration_language_list(self):
        result = [
            PyThaiNLPTransliterationLanguage(PyThaiNLPTransliterationMode.Romanization),
            PyThaiNLPTransliterationLanguage(PyThaiNLPTransliterationMode.IPA)
        ]
        return result

    def get_transliteration(self, text, transliteration_key):
        # todo: support IPA
        mode = PyThaiNLPTransliterationMode[transliteration_key['mode']]

        if mode ==  PyThaiNLPTransliterationMode.Romanization:
            return pythainlp.romanize(text, engine='thai2rom')
        elif mode == PyThaiNLPTransliterationMode.IPA:
            return pythainlp.transliterate(text)

    def get_tokenization(self, text, tokenization_key):
        mode = PyThaiNLPTokenizationMode[tokenization_key['mode']]
        
        if mode == PyThaiNLPTokenizationMode.Default:
            tokens = pythainlp.word_tokenize(text)
            tokens = [token for token in tokens if not token.isspace()]
            tokens = [token for token in tokens if not token in string.punctuation]
            token_entries = [{'token': token, 'lemma': token, 'can_translate': True, 'can_transliterate': True} for token in tokens]
            return token_entries

        # raise exception
        raise cloudlanguagetools.errors.RequestError(f'unsupported tokenization mode: {mode.name}')


    def get_tokenization_options(self):
        result = [
            PyThaiNLPTokenization(PyThaiNLPTokenizationMode.Default)
        ]
        return result


import spacy
import logging
import os

import cloudlanguagetools.service
import cloudlanguagetools.constants
import cloudlanguagetools.languages
import cloudlanguagetools.tokenization

class SpacyTokenization(cloudlanguagetools.tokenization.Tokenization):
    def __init__(self, language, model_name, variant=None):
        self.language = language
        self.service = cloudlanguagetools.constants.Service.Spacy
        self.model_name = model_name
        self.variant = variant

    def get_tokenization_name(self):
        if self.variant != None:
            result = f'{self.language.lang_name} ({self.variant}), {self.service.name}'
        else:
            result = f'{self.language.lang_name}, {self.service.name}'
        return result

    def get_tokenization_key(self):
        return {
            'model_name': self.model_name
        }

class SpacyService(cloudlanguagetools.service.Service):
    def __init__(self):
        self.nlp_engine_cache = {}
        os.environ["TOKENIZERS_PARALLELISM"] = "false"

    def preload_data(self):
        # pre-load nlp engines (to test ram utilization)
        for tokenization_option in self.get_tokenization_options():
            model_name = tokenization_option.get_tokenization_key()['model_name']
            nlp_engine = self.get_nlp_engine(model_name)
        

    def build_nlp_engine(self, model_name):
        import spacy.lang.zh
        if model_name == 'chinese_char':
            return spacy.lang.zh.Chinese()
        if model_name == 'chinese_jieba':
            return spacy.lang.zh.Chinese.from_config({"nlp": {"tokenizer": {"segmenter": "jieba"}}})
        if model_name == 'chinese_pkuseg':
            return spacy.lang.zh.Chinese.from_config({"nlp": {"tokenizer": {"segmenter": "pkuseg"}}})
        return spacy.load(model_name)        

    def get_nlp_engine(self, model_name):
        if model_name in self.nlp_engine_cache:
            return self.nlp_engine_cache[model_name]

        logging.info(f'loading model {model_name}')
        nlp_engine = self.build_nlp_engine(model_name)
        self.nlp_engine_cache[model_name] = nlp_engine
        return nlp_engine

    def get_tokenization(self, text, tokenization_key):
        model_name = tokenization_key['model_name']
        nlp_engine = self.get_nlp_engine(model_name)

        doc = nlp_engine(text)
        result = []
        for token in doc:
            lemma = token.lemma_
            if len(lemma) == 0:
                lemma = str(token)
            pos_description = spacy.explain(token.tag_)
            entry = {
                'token': str(token),
                'lemma': lemma,
                'can_translate': token.is_alpha,
                'can_transliterate': token.is_alpha
            }
            if pos_description != None:
                entry['pos_description'] = pos_description
            result.append(entry)
        return result

        # raise exception
        raise cloudlanguagetools.errors.RequestError(f'unsupported tokenization mode: {mode.name}')


    def get_tokenization_options(self):
        result = [
            SpacyTokenization(cloudlanguagetools.languages.Language.en, 'en_core_web_trf'),
            SpacyTokenization(cloudlanguagetools.languages.Language.fr, 'fr_dep_news_trf'),
            SpacyTokenization(cloudlanguagetools.languages.Language.ja, 'ja_core_news_lg'),
            SpacyTokenization(cloudlanguagetools.languages.Language.de, 'de_dep_news_trf'),
            SpacyTokenization(cloudlanguagetools.languages.Language.es, 'es_dep_news_trf'),
            SpacyTokenization(cloudlanguagetools.languages.Language.ru, 'ru_core_news_lg'),
            SpacyTokenization(cloudlanguagetools.languages.Language.pl, 'pl_core_news_lg'),
            SpacyTokenization(cloudlanguagetools.languages.Language.it, 'it_core_news_lg'),

        ]
        chinese_language_list = [
            cloudlanguagetools.languages.Language.zh_cn,
            cloudlanguagetools.languages.Language.zh_tw,
            cloudlanguagetools.languages.Language.yue
        ]
        for language in chinese_language_list:
            # chinese variants
            result.extend([
                SpacyTokenization(language, 'chinese_char', 'Characters'),
                SpacyTokenization(language, 'chinese_jieba', 'Jieba (words)'),
                SpacyTokenization(language, 'chinese_jieba', 'PKUSeg (words)')
            ])
        return result


import os
import json
import cryptography
import cryptography.fernet
import cloudlanguagetools.keys

SERVICES_CONFIGURATION_JSON = 'services_configuration.json'

def get_key():
    return os.environ['CLOUDLANGUAGETOOLS_CORE_KEY']

def encrypt():
    # open json file
    f = open(SERVICES_CONFIGURATION_JSON)
    config = json.load(f)
    f.close()    
    config_str = json.dumps(config)

    # encrypt    
    f = cryptography.fernet.Fernet(get_key())
    token = f.encrypt(config_str.encode('utf-8'))

    # write to python module
    keys_module_path = os.path.join(os.path.dirname(__file__), 'keys.py')
    f = open(keys_module_path, 'w')
    f.write(f"KEYS='{token.decode('utf-8')}'")
    f.close()

def decrypt():
    f = cryptography.fernet.Fernet(get_key())
    decoded_bytes = f.decrypt(cloudlanguagetools.keys.KEYS.encode('utf-8'))
    decoded_str = decoded_bytes.decode('utf-8')
    return json.loads(decoded_str)

def decrypt_write_json():
    f = open(SERVICES_CONFIGURATION_JSON, 'w')
    json.dump(decrypt(), f, indent=4)
    f.close()



from setuptools import setup
from setuptools.command.install import install

# build instructions
# python setup.py sdist
# twine upload dist/*

setup(name='cloudlanguagetools',
      version='1.6',
      description='Interface with various cloud APIs for language processing such as translation, text to speech',
      long_description=open('README.rst', encoding='utf-8').read(),
      url='https://github.com/Language-Tools/cloud-language-tools-core',
      author='Luc',
      author_email='languagetools@mailc.net',
      classifiers=[
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Text Processing :: Linguistic',
      ],      
      license='GPL',
      packages=['cloudlanguagetools'],
      install_requires=[
          'clt_spacy',
          'clt_argostranslate',
          'clt_requirements'
      ],
      )


import json
import requests
import tempfile
import logging
import time

import cloudlanguagetools.service
import cloudlanguagetools.constants
import cloudlanguagetools.languages
import cloudlanguagetools.ttsvoice
import cloudlanguagetools.translationlanguage
import cloudlanguagetools.transliterationlanguage
import cloudlanguagetools.errors


FPTAI_VOICE_SPEED_DEFAULT = 0


class FptAiVoice(cloudlanguagetools.ttsvoice.TtsVoice):
    def __init__(self, audio_language, voice_id, name, gender, region):
        self.service = cloudlanguagetools.constants.Service.FptAi
        self.audio_language = audio_language
        self.voice_id = voice_id
        self.name = name
        self.gender = gender
        self.region = region

    def get_voice_key(self):
        return {
            'voice_id': self.voice_id
        }

    def get_voice_shortname(self):
        return f'{self.name} ({self.region})'

    def get_options(self):
        return {
            'speed' : {
                'type': 'number',
                'min': -3,
                'max': 3,
                'default': FPTAI_VOICE_SPEED_DEFAULT
            },            
        }


class FptAiService(cloudlanguagetools.service.Service):
    def __init__(self):
        pass

    def configure(self, config):
        self.api_key = config['key']

    def get_translation(self, text, from_language_key, to_language_key):
        raise cloudlanguagetools.errors.RequestError('not supported')


    def get_tts_audio(self, text, voice_key, options):
        output_temp_file = tempfile.NamedTemporaryFile()
        output_temp_filename = output_temp_file.name

        api_url = "https://api.fpt.ai/hmi/tts/v5"
        body = text
        headers = {
            'api_key': self.api_key,
            'voice': voice_key['voice_id'],
            'Cache-Control': 'no-cache',
            'format': 'mp3',
            #'speed': str(options['speed'])
        }
        speed = options.get('speed', FPTAI_VOICE_SPEED_DEFAULT)
        if speed != FPTAI_VOICE_SPEED_DEFAULT:
            headers['speed'] = str(speed)
        response = requests.post(api_url, headers=headers, data=body.encode('utf-8'), 
            timeout=cloudlanguagetools.constants.RequestTimeout)

        if response.status_code == 200:
            response_data = response.json()
            async_url = response_data['async']
            logging.debug(f'received async_url: {async_url}')

            # wait until the audio is available
            audio_available = False
            total_tries = 7
            max_tries = total_tries
            wait_time = 0.2
            while audio_available == False and max_tries > 0:
                time.sleep(wait_time)
                logging.debug(f'checking whether audio is available on {async_url}')
                response = requests.get(async_url, allow_redirects=True, timeout=cloudlanguagetools.constants.RequestTimeout)
                if response.status_code == 200 and len(response.content) > 0:
                    with open(output_temp_filename, 'wb') as audio:
                        audio.write(response.content)                    
                    audio_available = True
                wait_time = wait_time * 2
                max_tries -= 1            
            
            if not audio_available:
                error_message = f'could not retrieve audio after {total_tries} tries (url {async_url})'
                raise cloudlanguagetools.errors.RequestError(error_message)

            return output_temp_file

        error_message = f'could not retrieve FPT.AI audio: {response.content}'
        raise cloudlanguagetools.errors.RequestError(error_message)


    def get_tts_voice_list(self):
        # returns list of TtSVoice
        return [
            FptAiVoice(cloudlanguagetools.languages.AudioLanguage.vi_VN, 'leminh', 'Lê Minh', cloudlanguagetools.constants.Gender.Male, 'miền Bắc'),
            FptAiVoice(cloudlanguagetools.languages.AudioLanguage.vi_VN, 'banmai', 'Ban Mai', cloudlanguagetools.constants.Gender.Female, 'miền Bắc'),
            FptAiVoice(cloudlanguagetools.languages.AudioLanguage.vi_VN, 'thuminh', 'Thu Minh', cloudlanguagetools.constants.Gender.Female, 'miền Bắc'),
            FptAiVoice(cloudlanguagetools.languages.AudioLanguage.vi_VN, 'giahuy', 'Gia Huy', cloudlanguagetools.constants.Gender.Male, 'miền Trung'),
            FptAiVoice(cloudlanguagetools.languages.AudioLanguage.vi_VN, 'ngoclam', 'Ngọc Lam', cloudlanguagetools.constants.Gender.Female, 'miền Trung'),
            FptAiVoice(cloudlanguagetools.languages.AudioLanguage.vi_VN, 'myan', 'Mỹ An', cloudlanguagetools.constants.Gender.Female, 'miền Trung'),
            FptAiVoice(cloudlanguagetools.languages.AudioLanguage.vi_VN, 'lannhi', 'Lan Nhi', cloudlanguagetools.constants.Gender.Female, 'miền Nam'),
            FptAiVoice(cloudlanguagetools.languages.AudioLanguage.vi_VN, 'linhsan', 'Linh San', cloudlanguagetools.constants.Gender.Female, 'miền Nam'),
            FptAiVoice(cloudlanguagetools.languages.AudioLanguage.vi_VN, 'minhquang', 'Minh Quang', cloudlanguagetools.constants.Gender.Male, 'miền Nam'),
            # acesound voices
            FptAiVoice(cloudlanguagetools.languages.AudioLanguage.vi_VN, 'banmaiace', 'Ban Mai (AceSound)', cloudlanguagetools.constants.Gender.Female, 'miền Bắc'),
            FptAiVoice(cloudlanguagetools.languages.AudioLanguage.vi_VN, 'thuminhace', 'Thu Minh (AceSound)', cloudlanguagetools.constants.Gender.Female, 'miền Bắc'),
            FptAiVoice(cloudlanguagetools.languages.AudioLanguage.vi_VN, 'ngoclamace', 'Ngọc Lam (AceSound)', cloudlanguagetools.constants.Gender.Female, 'miền Trung'),
            FptAiVoice(cloudlanguagetools.languages.AudioLanguage.vi_VN, 'linhsanace', 'Linh San (AceSound)', cloudlanguagetools.constants.Gender.Female, 'miền Nam'),
            FptAiVoice(cloudlanguagetools.languages.AudioLanguage.vi_VN, 'minhquangace', 'Minh Quang (AceSound)', cloudlanguagetools.constants.Gender.Male, 'miền Nam'),


        ]

    def get_translation_language_list(self):
        result = []
        return result

    def get_transliteration_language_list(self):
        result = []
        return result
   


import enum

# usage/account related constants 
# ===============================

class RequestType(enum.Enum):
    translation = enum.auto()
    transliteration = enum.auto()
    audio = enum.auto()
    breakdown = enum.auto()

class UsageScope(enum.Enum):
    def __init__(self, key_str):
        self.key_str = key_str 
    User = ("user")
    Global = ("global")

class UsagePeriod(enum.Enum):
    daily = enum.auto()
    monthly = enum.auto()
    patreon_monthly = enum.auto()
    lifetime = enum.auto()
    recurring = enum.auto()

class ApiKeyType(enum.Enum):
    test = enum.auto()
    patreon = enum.auto()
    trial = enum.auto()
    getcheddar = enum.auto()

class Client(enum.Enum):
    awesometts = enum.auto()
    languagetools = enum.auto()
    hypertts = enum.auto()
    test = enum.auto()

# what triggered this request (batch / on the fly / editor)
class RequestMode(enum.Enum):
    batch = enum.auto()
    dynamic = enum.auto()
    edit = enum.auto()

# service and language related constants
# ======================================

RequestTimeout = 10 # 10 seconds max
ReadTimeout = 3 # 3 seconds read timeout

class Service(enum.Enum):
    Azure = enum.auto()
    Google = enum.auto()
    MandarinCantonese = enum.auto()
    EasyPronunciation = enum.auto()
    Watson = enum.auto()
    Naver = enum.auto()
    Amazon = enum.auto()
    Forvo = enum.auto()
    CereProc = enum.auto()
    Epitran = enum.auto()
    DeepL = enum.auto()
    VocalWare = enum.auto()
    Voicen = enum.auto()
    FptAi = enum.auto()
    PyThaiNLP = enum.auto()
    Spacy = enum.auto()
    ArgosTranslate = enum.auto()

class Gender(enum.Enum):
    Male = enum.auto()
    Female = enum.auto()
    Any = enum.auto()




import json
import requests
import urllib
import tempfile
import logging
import os
import pprint

import cloudlanguagetools.service
import cloudlanguagetools.constants
import cloudlanguagetools.languages
import cloudlanguagetools.ttsvoice
import cloudlanguagetools.translationlanguage
import cloudlanguagetools.transliterationlanguage
import cloudlanguagetools.errors

GENDER_MAP = {
    cloudlanguagetools.constants.Gender.Male: 'm',
    cloudlanguagetools.constants.Gender.Female: 'f'
}

COUNTRY_ANY = 'ANY'

class ForvoVoice(cloudlanguagetools.ttsvoice.TtsVoice):
    def __init__(self, language_code, country_code, audio_language, gender):
        # print(voice_data)
        self.service = cloudlanguagetools.constants.Service.Forvo
        self.language_code = language_code
        self.country_code = country_code
        self.audio_language = audio_language
        self.gender = gender

    def get_voice_key(self):
        result = {
            'language_code': self.language_code,
            'country_code': self.country_code
        }
        if self.gender != cloudlanguagetools.constants.Gender.Any:
            result['gender'] = GENDER_MAP[self.gender]
        return result

    def get_voice_description(self):
        return f'{self.get_audio_language_name()}, {self.get_gender().name}, {self.service.name}'

    def get_voice_shortname(self):
        return None

    def get_options(self):
        return {}



class ForvoService(cloudlanguagetools.service.Service):
    def __init__(self):
        self.url_base = 'https://apicommercial.forvo.com'
        self.build_audio_language_map()

    def configure(self, config):
        self.key = config['key']

    def get_headers(self):
        # forvo uses cloudflare or something equivalent
        return {'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:85.0) Gecko/20100101 Firefox/85.0'}

    def get_tts_audio(self, text, voice_key, options):

        language = voice_key['language_code']

        sex_param = ''
        if 'gender' in voice_key:
            sex_param = f"/sex/{voice_key['gender']}"
        
        country_code = ''
        if voice_key['country_code'] != COUNTRY_ANY:
            # user selected a particular country
            country_code = f"/country/{voice_key['country_code']}"

        username_param = ''
        if 'preferred_user' in voice_key:
            username_param = f"/username/{voice_key['preferred_user']}"

        encoded_text = urllib.parse.quote(text)

        url = f'{self.url_base}/key/{self.key}/format/json/action/word-pronunciations/word/{encoded_text}/language/{language}{sex_param}{username_param}/order/rate-desc/limit/1{country_code}'

        try:
            response = requests.get(url, headers=self.get_headers(), timeout=cloudlanguagetools.constants.RequestTimeout)
            if response.status_code == 200:
                data = response.json()
                items = data['items']
                if len(items) == 0:
                    error_message = f"Pronunciation not found in Forvo for word [{text}], language={language}, country={voice_key['country_code']}"
                    raise cloudlanguagetools.errors.NotFoundError(error_message)
                audio_url = items[0]['pathmp3']
                output_temp_file = tempfile.NamedTemporaryFile()
                output_temp_filename = output_temp_file.name
                audio_request = requests.get(audio_url, headers=self.get_headers(), timeout=cloudlanguagetools.constants.RequestTimeout)
                open(output_temp_filename, 'wb').write(audio_request.content)
                return output_temp_file
            else:
                error_message = f'status_code: {response.status_code} response: {response.content}'
                raise cloudlanguagetools.errors.RequestError(error_message)
        except requests.exceptions.ReadTimeout as exception:
            raise cloudlanguagetools.errors.TimeoutError(f'timeout while retrieving forvo audio')


    def get_language_enum(self, language_id):
        forvo_language_id_map = {
            'zh': 'zh_cn',
            'ind': 'id_',
            'pt': 'pt_pt'
        }
        if language_id in forvo_language_id_map:
            language_id = forvo_language_id_map[language_id]
        return cloudlanguagetools.languages.Language[language_id]

    def get_audio_language_enum(self, language_id):
        pass

    def get_country_code(self, audio_language):
        # https://en.wikipedia.org/wiki/ISO_3166-1
        country_code_map = {
            cloudlanguagetools.languages.AudioLanguage.fr_FR: 'FRA',
            cloudlanguagetools.languages.AudioLanguage.fr_CH: 'CHE',
            cloudlanguagetools.languages.AudioLanguage.fr_BE: 'BEL',
            cloudlanguagetools.languages.AudioLanguage.nl_BE: 'BEL',
            cloudlanguagetools.languages.AudioLanguage.nl_NL: 'NLD',
            cloudlanguagetools.languages.AudioLanguage.de_AT: 'AUT',
            cloudlanguagetools.languages.AudioLanguage.de_DE: 'DEU',
            cloudlanguagetools.languages.AudioLanguage.de_CH: 'CHE',
            cloudlanguagetools.languages.AudioLanguage.en_AU: 'AUS',
            cloudlanguagetools.languages.AudioLanguage.en_CA: 'CAN',
            cloudlanguagetools.languages.AudioLanguage.en_GB: 'GBR',
            cloudlanguagetools.languages.AudioLanguage.en_IE: 'IRL',
            cloudlanguagetools.languages.AudioLanguage.en_IN: 'IND',
            cloudlanguagetools.languages.AudioLanguage.en_HK: 'HKG',
            cloudlanguagetools.languages.AudioLanguage.en_US: 'USA',
            cloudlanguagetools.languages.AudioLanguage.en_PH: 'PHL',
            cloudlanguagetools.languages.AudioLanguage.en_NZ: 'NZL',
            cloudlanguagetools.languages.AudioLanguage.en_SG: 'SGP',
            cloudlanguagetools.languages.AudioLanguage.en_ZA: 'ZAF',

            cloudlanguagetools.languages.AudioLanguage.en_GB_WLS: 'GBR', 
            cloudlanguagetools.languages.AudioLanguage.bn_BD: 'BGD', 
            cloudlanguagetools.languages.AudioLanguage.en_KE: 'KEN', 
            cloudlanguagetools.languages.AudioLanguage.ta_IN: 'IND', 
            cloudlanguagetools.languages.AudioLanguage.ur_IN: 'IND',
            cloudlanguagetools.languages.AudioLanguage.bn_IN: 'IND',
            cloudlanguagetools.languages.AudioLanguage.en_NG: 'NGA',
            cloudlanguagetools.languages.AudioLanguage.ta_LK: 'LKA',
            cloudlanguagetools.languages.AudioLanguage.ur_PK: 'PAK',
            cloudlanguagetools.languages.AudioLanguage.en_TZ: 'TZA',
            cloudlanguagetools.languages.AudioLanguage.ta_SG: 'SGP',
            cloudlanguagetools.languages.AudioLanguage.ta_MY: 'MYS',




            # arabic
            cloudlanguagetools.languages.AudioLanguage.ar_AE: 'ARE',
            cloudlanguagetools.languages.AudioLanguage.ar_BH: 'BHR',
            cloudlanguagetools.languages.AudioLanguage.ar_DZ: 'DZA',
            cloudlanguagetools.languages.AudioLanguage.ar_EG: 'EGY',
            cloudlanguagetools.languages.AudioLanguage.ar_IQ: 'IRQ',
            cloudlanguagetools.languages.AudioLanguage.ar_JO: 'JOR',
            cloudlanguagetools.languages.AudioLanguage.ar_KW: 'KWT',
            cloudlanguagetools.languages.AudioLanguage.ar_LY: 'LBY',
            cloudlanguagetools.languages.AudioLanguage.ar_MA: 'MAR',
            cloudlanguagetools.languages.AudioLanguage.ar_SA: 'SAU',
            cloudlanguagetools.languages.AudioLanguage.ar_QA: 'QAT',
            cloudlanguagetools.languages.AudioLanguage.ar_SA: 'SAU',
            cloudlanguagetools.languages.AudioLanguage.ar_SY: 'SYR',
            cloudlanguagetools.languages.AudioLanguage.ar_TN: 'TUN',
            cloudlanguagetools.languages.AudioLanguage.ar_XA: 'ANY', # any country
            cloudlanguagetools.languages.AudioLanguage.ar_YE: 'YEM', 
            cloudlanguagetools.languages.AudioLanguage.ar_LB: 'LBN', 
            cloudlanguagetools.languages.AudioLanguage.ar_OM: 'OMN', 

            # spanish
            cloudlanguagetools.languages.AudioLanguage.es_AR: 'ARG', 
            cloudlanguagetools.languages.AudioLanguage.es_BO: 'BOL',
            cloudlanguagetools.languages.AudioLanguage.es_CL: 'CHL',
            cloudlanguagetools.languages.AudioLanguage.es_CO: 'COL',
            cloudlanguagetools.languages.AudioLanguage.es_CR: 'CRI',
            cloudlanguagetools.languages.AudioLanguage.es_CU: 'CUB',
            cloudlanguagetools.languages.AudioLanguage.es_DO: 'DOM',
            cloudlanguagetools.languages.AudioLanguage.es_EC: 'ECU',
            cloudlanguagetools.languages.AudioLanguage.es_ES: 'ESP',
            cloudlanguagetools.languages.AudioLanguage.es_GQ: 'GNQ',
            cloudlanguagetools.languages.AudioLanguage.es_GT: 'GTM',
            cloudlanguagetools.languages.AudioLanguage.es_HN: 'HND',
            cloudlanguagetools.languages.AudioLanguage.es_LA: 'ANY', # any country
            cloudlanguagetools.languages.AudioLanguage.es_MX: 'MEX',
            cloudlanguagetools.languages.AudioLanguage.es_NI: 'NIC',
            cloudlanguagetools.languages.AudioLanguage.es_PA: 'PAN',
            cloudlanguagetools.languages.AudioLanguage.es_PE: 'PER',
            cloudlanguagetools.languages.AudioLanguage.es_PR: 'PRI',
            cloudlanguagetools.languages.AudioLanguage.es_PY: 'PRY',
            cloudlanguagetools.languages.AudioLanguage.es_SV: 'SLV',
            cloudlanguagetools.languages.AudioLanguage.es_US: 'USA', 
            cloudlanguagetools.languages.AudioLanguage.es_UY: 'URY', 
            cloudlanguagetools.languages.AudioLanguage.es_VE: 'VEN', 

            cloudlanguagetools.languages.AudioLanguage.ba_RU: 'RUS',
            cloudlanguagetools.languages.AudioLanguage.eu: 'ESP',
            cloudlanguagetools.languages.AudioLanguage.en_CB: 'VGB',
        }
        if audio_language not in country_code_map:
            logging.error(f'no country code found for {audio_language}')
        return country_code_map[audio_language]

    def get_voices_for_language_entry(self, language):
        try:
            language_code = language['code']
            language_enum = self.get_language_enum(language_code)
            # create as many voices as there are audio languages available

            audio_language_list = self.audio_language_map[language_enum]
            
            voices = []

            for gender in cloudlanguagetools.constants.Gender:
                if len(audio_language_list) == 1:
                    country_code = COUNTRY_ANY
                    voices.append(ForvoVoice(language_code, country_code, audio_language_list[0], gender))
                else:
                    # logging.info(f'multiple audio languages found: {audio_language_list}')
                    for audio_language in audio_language_list:
                        country_code = self.get_country_code(audio_language)
                        voices.append(ForvoVoice(language_code, country_code, audio_language, gender))

            return voices

        except KeyError:
            logging.warn(f'forvo language mapping not found: {language}')

        return []

    def build_audio_language_map(self):
        self.audio_language_map = {}
        for audio_language in cloudlanguagetools.languages.AudioLanguage:
            language = audio_language.lang
            if language not in self.audio_language_map:
                self.audio_language_map[language] = []
            self.audio_language_map[language].append(audio_language)
            

    def get_tts_voice_list(self):
        # returns list of TtSVoice

        voice_list = []

        # https://api.forvo.com/documentation/word-pronunciations/
        url = f'{self.url_base}/key/{self.key}/format/json/action/language-list/min-pronunciations/5000'
        response = requests.get(url, headers=self.get_headers(), timeout=cloudlanguagetools.constants.RequestTimeout)
        if response.status_code == 200:
            data = response.json()
            languages = data['items']
            for language in languages:
                voice_list.extend(self.get_voices_for_language_entry(language))

            # pprint.pprint(data)
        else:
            print(response.content)

        return voice_list


    def get_translation_language_list(self):
        result = []
        return result

    def get_transliteration_language_list(self):
        result = []
        return result


    


import json
import requests
import tempfile
import logging
import uuid
import operator
import pydub

import cloudlanguagetools.service
import cloudlanguagetools.constants
import cloudlanguagetools.languages
import cloudlanguagetools.ttsvoice
import cloudlanguagetools.translationlanguage
import cloudlanguagetools.transliterationlanguage
import cloudlanguagetools.errors

NAVER_VOICE_SPEED_DEFAULT = 0
NAVER_VOICE_PITCH_DEFAULT = 0

class NaverVoice(cloudlanguagetools.ttsvoice.TtsVoice):
    def __init__(self, audio_language, name, gender, description, voice_type):
        self.service = cloudlanguagetools.constants.Service.Naver
        self.audio_language = audio_language
        self.name = name
        self.gender = gender
        self.description = description
        self.voice_type = voice_type

    def get_voice_key(self):
        return {
            'name': self.name
        }

    def get_voice_shortname(self):
        return f'{self.description} ({self.voice_type})'

    def get_options(self):
        return {
            'speed' : {
                'type': 'number_int',
                'min': -5,
                'max': 5,
                'default': NAVER_VOICE_SPEED_DEFAULT
            },
            'pitch': {
                'type': 'number_int',
                'min': -5,
                'max': 5,
                'default': NAVER_VOICE_PITCH_DEFAULT
            }
        }

class NaverTranslationLanguage(cloudlanguagetools.translationlanguage.TranslationLanguage):
    def __init__(self, language, language_id):
        self.service = cloudlanguagetools.constants.Service.Naver
        self.language = language
        self.language_id = language_id

    def get_language_id(self):
        return self.language_id

class NaverService(cloudlanguagetools.service.Service):
    def __init__(self):
        pass

    def configure(self, config):
        self.client_id = config['client_id']
        self.client_secret = config['client_secret']

    def get_translation(self, text, from_language_key, to_language_key):
        url = 'https://naveropenapi.apigw.ntruss.com/nmt/v1/translation'
        headers = {
            'X-NCP-APIGW-API-KEY-ID': self.client_id,
            'X-NCP-APIGW-API-KEY': self.client_secret
        }

        data = {
            'text': text,
            'source': from_language_key,
            'target': to_language_key
        }

        # alternate_data = 'speaker=clara&text=vehicle&volume=0&speed=0&pitch=0&format=mp3'
        response = requests.post(url, json=data, headers=headers, timeout=cloudlanguagetools.constants.RequestTimeout)
        if response.status_code == 200:
            response_data = response.json()
            return response_data['message']['result']['translatedText']

        error_message = f'Status code: {response.status_code}: {response.content}'
        raise cloudlanguagetools.errors.RequestError(error_message)


    def get_tts_audio(self, text, voice_key, options):
        output_temp_file = tempfile.NamedTemporaryFile()
        output_temp_filename = output_temp_file.name

        url = 'https://naveropenapi.apigw.ntruss.com/tts-premium/v1/tts'
        headers = {
            'Content-Type': 'application/x-www-form-urlencoded',
            'X-NCP-APIGW-API-KEY-ID': self.client_id,
            'X-NCP-APIGW-API-KEY': self.client_secret
        }

        data = {
            'text': text,
            'speaker': voice_key['name'],
            'speed': options.get('speed', NAVER_VOICE_SPEED_DEFAULT),
            'pitch': options.get('pitch', NAVER_VOICE_PITCH_DEFAULT)
        }

        # alternate_data = 'speaker=clara&text=vehicle&volume=0&speed=0&pitch=0&format=mp3'
        response = requests.post(url, data=data, headers=headers, timeout=cloudlanguagetools.constants.RequestTimeout)
        if response.status_code == 200:
            with open(output_temp_filename, 'wb') as audio:
                audio.write(response.content)
            return output_temp_file

        response_data = response.json()
        error_message = f'Status code: {response.status_code}: {response_data}'
        raise cloudlanguagetools.errors.RequestError(error_message)

    def get_tts_voice_list(self):
        # returns list of TtSVoice
        return [
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'mijin', cloudlanguagetools.constants.Gender.Female, 'Mijin', 'General'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'jinho', cloudlanguagetools.constants.Gender.Male, 'Jinho', 'General'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.en_US, 'clara', cloudlanguagetools.constants.Gender.Female, 'Clara', 'General'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.en_US, 'matt', cloudlanguagetools.constants.Gender.Male, 'Matt', 'General'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ja_JP, 'shinji', cloudlanguagetools.constants.Gender.Male, 'Shinji', 'General'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.zh_CN, 'meimei', cloudlanguagetools.constants.Gender.Female, 'Meimei', 'General'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.zh_CN, 'liangliang', cloudlanguagetools.constants.Gender.Male, 'Liangliang', 'General'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.es_ES, 'jose', cloudlanguagetools.constants.Gender.Male, 'Jose', 'General'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.es_ES, 'carmen', cloudlanguagetools.constants.Gender.Female, 'Carmen', 'General'),

            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'dara', cloudlanguagetools.constants.Gender.Female, 'Dara', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'nara', cloudlanguagetools.constants.Gender.Female, 'Nara', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'nminsang', cloudlanguagetools.constants.Gender.Male, 'Minsang', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'nhajun', cloudlanguagetools.constants.Gender.Male, 'Hajoon', 'Premium (Child)'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'ndain', cloudlanguagetools.constants.Gender.Female, 'Dain', 'Premium (Child)'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'njiyun', cloudlanguagetools.constants.Gender.Female, 'Jiyoon', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'nsujin', cloudlanguagetools.constants.Gender.Female, 'Sujin', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'njinho', cloudlanguagetools.constants.Gender.Male, 'Jinho', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'nsinu', cloudlanguagetools.constants.Gender.Male, 'Shinwoo', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'njihun', cloudlanguagetools.constants.Gender.Male, 'Jihoon', 'Premium'),

            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ja_JP, 'ntomoko', cloudlanguagetools.constants.Gender.Female, 'Tomoko', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ja_JP, 'nnaomi', cloudlanguagetools.constants.Gender.Female, 'Naomi', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ja_JP, 'dnaomi_joyful', cloudlanguagetools.constants.Gender.Female, 'Naomi (Joyful)', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ja_JP, 'dnaomi_formal', cloudlanguagetools.constants.Gender.Female, 'Naomi (Formal)', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ja_JP, 'nsayuri', cloudlanguagetools.constants.Gender.Female, 'Sayuri', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ja_JP, 'driko', cloudlanguagetools.constants.Gender.Female, 'Riko', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ja_JP, 'deriko', cloudlanguagetools.constants.Gender.Female, 'Eriko', 'Premium'),

            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'ngoeun', cloudlanguagetools.constants.Gender.Female, 'Koeun', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'neunyoung', cloudlanguagetools.constants.Gender.Female, 'Eunyoung', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'nsunkyung', cloudlanguagetools.constants.Gender.Female, 'Sunkyung', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'nyujin', cloudlanguagetools.constants.Gender.Female, 'Yujin', 'Premium'),
            
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'ntaejin', cloudlanguagetools.constants.Gender.Male, 'Taejin', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'nyoungil', cloudlanguagetools.constants.Gender.Male, 'Youngil', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'nseungpyo', cloudlanguagetools.constants.Gender.Male, 'Seungpyo', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'nwontak', cloudlanguagetools.constants.Gender.Male, 'Wontak', 'Premium'),
            
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'dara_ang', cloudlanguagetools.constants.Gender.Female, 'Ara (Angry)', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'nsunhee', cloudlanguagetools.constants.Gender.Female, 'Sunhee', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'nminseo', cloudlanguagetools.constants.Gender.Female, 'Minseo', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'njiwon', cloudlanguagetools.constants.Gender.Female, 'Jiwon', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'nbora', cloudlanguagetools.constants.Gender.Female, 'Bora', 'Premium'),
            
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'njonghyun', cloudlanguagetools.constants.Gender.Male, 'Jonghyun', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'njoonyoung', cloudlanguagetools.constants.Gender.Male, 'Joonyoung', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'njaewook', cloudlanguagetools.constants.Gender.Male, 'Jaewook', 'Premium'),
            
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.en_US, 'danna', cloudlanguagetools.constants.Gender.Female, 'Anna', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.en_US, 'djoey', cloudlanguagetools.constants.Gender.Female, 'Joey', 'Premium'),
            
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ja_JP, 'dhajime', cloudlanguagetools.constants.Gender.Male, 'Hajime', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ja_JP, 'ddaiki', cloudlanguagetools.constants.Gender.Male, 'Daiki', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ja_JP, 'dayumu', cloudlanguagetools.constants.Gender.Male, 'Ayumu', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ja_JP, 'dmio', cloudlanguagetools.constants.Gender.Female, 'Mio', 'Premium'),

            NaverVoice(cloudlanguagetools.languages.AudioLanguage.zh_TW, 'chiahua', cloudlanguagetools.constants.Gender.Female, 'Chiahua', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.zh_TW, 'kuanlin', cloudlanguagetools.constants.Gender.Male, 'Kuanlin', 'Premium'),

            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'nes_c_hyeri', cloudlanguagetools.constants.Gender.Female, 'Hyeri', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'nes_c_sohyun', cloudlanguagetools.constants.Gender.Female, 'Sohyun', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'nes_c_mikyung', cloudlanguagetools.constants.Gender.Female, 'Mikyung', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'nes_c_kihyo', cloudlanguagetools.constants.Gender.Female, 'Kihyo', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'ntiffany', cloudlanguagetools.constants.Gender.Female, 'Kiseo', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'napple', cloudlanguagetools.constants.Gender.Female, 'Neulbom', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'njangj', cloudlanguagetools.constants.Gender.Female, 'Dream', 'Premium'),
            NaverVoice(cloudlanguagetools.languages.AudioLanguage.ko_KR, 'noyj', cloudlanguagetools.constants.Gender.Female, 'Bomdal', 'Premium'),


        ]

    def get_translation_language_list(self):
        result = [
            NaverTranslationLanguage(cloudlanguagetools.languages.Language.ko, 'ko'),
            NaverTranslationLanguage(cloudlanguagetools.languages.Language.en, 'en'),
            NaverTranslationLanguage(cloudlanguagetools.languages.Language.ja, 'ja'),
            NaverTranslationLanguage(cloudlanguagetools.languages.Language.zh_cn, 'zh-CN'),
            NaverTranslationLanguage(cloudlanguagetools.languages.Language.zh_tw, 'zh-TW'),
            NaverTranslationLanguage(cloudlanguagetools.languages.Language.vi, 'vi'),
            NaverTranslationLanguage(cloudlanguagetools.languages.Language.id_, 'id'),
            NaverTranslationLanguage(cloudlanguagetools.languages.Language.th, 'th'),
            NaverTranslationLanguage(cloudlanguagetools.languages.Language.de, 'de'),
            NaverTranslationLanguage(cloudlanguagetools.languages.Language.ru, 'ru'),
            NaverTranslationLanguage(cloudlanguagetools.languages.Language.it, 'it'),
            NaverTranslationLanguage(cloudlanguagetools.languages.Language.fr, 'fr'),
        ]
        return result

    def get_transliteration_language_list(self):
        result = []
        return result
   


import os
import requests
import urllib.parse

import cloudlanguagetools.service
import cloudlanguagetools.constants
import cloudlanguagetools.languages
import cloudlanguagetools.transliterationlanguage

VARIANT_JAPANESE_ROMAJI = 'Romaji'
VARIANT_JAPANESE_KANA = 'Kana'

class EasyPronunciationTransliterationLanguage(cloudlanguagetools.transliterationlanguage.TransliterationLanguage):
    def __init__(self, url_path, language, api_params, api_key, variant = None):
        self.service = cloudlanguagetools.constants.Service.EasyPronunciation
        self.url_path = url_path
        self.language = language
        self.api_params = api_params
        self.api_key = api_key
        self.variant = variant

    def get_transliteration_name(self):
        result = f'{self.language.lang_name} (IPA Pronunciation), {self.service.name}'
        if self.variant != None:
            result = f'{self.language.lang_name} ({self.variant}), {self.service.name}'
        return result

    def get_transliteration_shortname(self):
        result = f'IPA Pronunciation, {self.service.name}'
        if self.variant != None:
            result = f'{self.variant}, {self.service.name}'
        return result        

    def get_transliteration_key(self):
        key = {
            'url_path': self.url_path,
            'api_params': self.api_params,
            'api_key': self.api_key,
        }
        if self.variant != None:
            key['variant'] = self.variant
        return key

class EasyPronunciationService(cloudlanguagetools.service.Service):
    def __init__(self):
        self.url_base = 'https://easypronunciation.com'

    def configure(self, config):
        self.api_keys = {
            'french': config['french_api_key'],
            'english': config['english_api_key'],
            'italian': config['italian_api_key'],
            'portuguese': config['portuguese_api_key'],
            'japanese': config['japanese_api_key'],
            'spanish': config['spanish_api_key'],
            'german': config['german_api_key'],
            'russian': config['russian_api_key']
        }

    def get_tts_voice_list(self):
        return []

    def get_translation_language_list(self):
        return []

    def get_transliteration_language_list(self):
        result = [
            EasyPronunciationTransliterationLanguage('/french-api.php', cloudlanguagetools.languages.Language.fr,
            {
                'version': 1,
                'use_un_symbol': 1,
                'vowel_lengthening':1,
                'show_rare_pronunciations':0,
                'french_liaison_styling': 'one_by_one',
                'spell_numbers':1
            }, 'french'),

            EasyPronunciationTransliterationLanguage('/english-api.php', cloudlanguagetools.languages.Language.en,
            {
                'version': 1,
                'english_phonetics_algorithm': 'american_miscellaneous_sources',
                'Convert_to_english':'ipa',
                'add_aspiration_symbol': 1,
                'narrow_transcription':1,
                'r_replacement':1,
                'cot_caught_merger':1,
                'er_replacement':1,
                'elongation_symbol_after_i_and_u':'always',
                'only_i_for_es_ed_endings':0,
                'show_rare_pronunciations':1,
                'spell_numbers':1
            }, 'english'),

            EasyPronunciationTransliterationLanguage('/italian-api.php', cloudlanguagetools.languages.Language.it,
            {
                'version':1,
                'spell_numbers':1
            }, 'italian'),

            EasyPronunciationTransliterationLanguage('/japanese-api.php', cloudlanguagetools.languages.Language.ja,
            {
                'version':1,
                'style':'slash',
                'devoicing':1,
                'weakening':1
            }, 'japanese', VARIANT_JAPANESE_ROMAJI),

            EasyPronunciationTransliterationLanguage('/japanese-api.php', cloudlanguagetools.languages.Language.ja,
            {
                'version':1,
                'style':'slash',
                'devoicing':1,
                'weakening':1
            }, 'japanese', VARIANT_JAPANESE_KANA),

            EasyPronunciationTransliterationLanguage('/portuguese-api.php', cloudlanguagetools.languages.Language.pt_pt,
            {
                'version':1,
                'portuguese_dialect':'EP',
                'spell_numbers':1,
            }, 'portuguese'),

            EasyPronunciationTransliterationLanguage('/portuguese-api.php', cloudlanguagetools.languages.Language.pt_br,
            {
                'version':1,
                'portuguese_dialect':'BP',
                'spell_numbers':1,
            }, 'portuguese'),

            EasyPronunciationTransliterationLanguage('/spanish-api.php', cloudlanguagetools.languages.Language.es,
            {
                'version':1,
                'split_into_syllables':0,
                'caza_casa_merger':1,
                'callo_cayo_merger':1,
                'allophones_for_l_m_n':1,
                'x_before_consonants':'s',
                'spell_numbers':1
            }, 'spanish'),

            EasyPronunciationTransliterationLanguage('/german-api.php', cloudlanguagetools.languages.Language.de,
            {
                'version':1,
                'split_into_syllables':1,
                'spell_numbers':1
            }, 'german'),

            EasyPronunciationTransliterationLanguage('/russian-api.php', cloudlanguagetools.languages.Language.ru,
            {
                'version':1,
                'Convert_to_russian':'IPA',
                'do_not_restore_yo': 0,
                'split_into_syllables':1,
                'spell_numbers':1
            }, 'russian'),

        ]
        return result

    def get_transliteration(self, text, transliteration_key):
        api_url = self.url_base + transliteration_key['url_path']
        parameters = {
            'access_token': self.api_keys[transliteration_key['api_key']],
            'phrase': text
        }
        parameters.update(transliteration_key['api_params'])
        encoded_parameters = urllib.parse.urlencode(parameters)
        full_url = f'{api_url}?{encoded_parameters}'

        # print(full_url)
        request = requests.get(full_url)
        result = request.json()

        # print(request)
        # print(result)

        if 'phonetic_transcription' in result:
            phonetic_transcription = result['phonetic_transcription']
            result_components = []
            for entry in phonetic_transcription:
                result_components.append(entry['transcriptions'][0])

            if 'variant' in transliteration_key:
                if transliteration_key['variant'] == VARIANT_JAPANESE_ROMAJI:
                    result_components = [x['romaji'] for x in result_components]
                if transliteration_key['variant'] == VARIANT_JAPANESE_KANA:
                    result_components = [x['kana'] for x in result_components]

            # print(result_components)
            return ' '.join(result_components)

        # an error occured
        error_message = f'EasyPronunciation: could not perform conversion: {str(result)}'
        raise cloudlanguagetools.errors.RequestError(error_message)




class TtsVoice():
    def __init__(self):
        pass

    def get_gender(self):
        return self.gender

    def get_language_code(self):
        return self.audio_language.lang.name

    def get_audio_language_code(self):
        return self.audio_language.name

    def get_audio_language_name(self):
        return self.audio_language.audio_lang_name

    def get_voice_description(self):
        return f'{self.get_audio_language_name()}, {self.get_gender().name}, {self.get_voice_shortname()}, {self.service.name}'

    def json_obj(self):
        return {
            'service': self.service.name,
            'gender': self.get_gender().name,
            'language_code': self.get_language_code(),
            'audio_language_code': self.get_audio_language_code(),
            'audio_language_name': self.get_audio_language_name(),
            'voice_key': self.get_voice_key(),
            'voice_description': self.get_voice_description(),
            'voice_name': self.get_voice_shortname(),
            'options': self.get_options()
        }
   
